@@ -7,6 +7,7 @@
 use core::cell::{Cell, RefCell};
 
 use cortex_m::{
+    delay::Delay,
     interrupt::{free, Mutex},
     peripheral::NVIC,
 };
@@ -15,10 +16,10 @@ use cortex_m_rt::entry;
 use stm32_hal2::{
     adc::{Adc, AdcChannel, AdcDevice, AdcInterrupt, Align, ClockMode, InputType, OperationMode},
     clocks::Clocks,
-    delay::Delay,
     dma::{self, Dma, DmaChannel, DmaInterrupt, DmaWriteBuf},
-    gpio::{Edge, PinMode, PinNum},
+    gpio::{Edge, PinMode},
     low_power, pac,
+    traits::ClockCfg,
 };
 
 #[entry]
@@ -30,20 +31,18 @@ fn main() -> ! {
 
     let clock_cfg = Clocks::default();
 
-    if clock_cfg.setup(&mut dp.RCC, &mut dp.FLASH).is_err() {
-        defmt::error!("Unable to configure clocks due to a speed error.")
-    };
+    clock_cfg.setup(&mut dp.RCC, &mut dp.FLASH).unwrap();
 
-    let mut delay = Delay::new(cp.SYST, &clock_cfg);
+    let mut delay = Delay::new(cp.SYST, clock_cfg.systick());
 
     let chan_num = 2;
 
     // Enable the GPIOB port.
-    let mut gpiob = GpioB::new(dp.GPIOB, &mut dp.RCC);
+    let mut gpiob = GpioB::new(dp.GPIOB);
 
     // Configure the ADC pin in analog mode. (This is the default state for some STM32 families,
     // but not all)
-    let _adc_pin = gpiob.new_pin(PinNum::P0, PinMode::Analog);
+    let _adc_pin = gpiob.new_pin(0, PinMode::Analog);
 
     let mut adc = Adc::new_adc1(
         dp.ADC1,
@@ -51,7 +50,6 @@ fn main() -> ! {
         &mut dp.ADC_COMMON,
         ClockMode::default(),
         &clock_cfg,
-        &mut dp.RCC,
     );
 
     // 1: Confiuration options:
@@ -81,7 +79,7 @@ fn main() -> ! {
     // Begin a DMA transfer. Note that the `DmaChannel` we pass here is only used on
     // MCUs that use `DMAMUX`, eg L5, G0, and G4. For those, you need to run `mux`, to
     // set the channel: `dma::mux(DmaChannel::C1, MuxInput::Adc1, &mut dp.DMAMUX);
-    adc.read_dma(&mut dma_buf, chan_num, DmaChannel::C1, &mut dma);
+    unsafe { adc.read_dma(&mut dma_buf, chan_num, DmaChannel::C1, &mut dma) };
 
     // Wait for the transfer to complete. Ie by handling the channel's transfer-complete
     // interrupt in an ISR, which is enabled by the `read_dma` command.
@@ -101,6 +99,10 @@ fn main() -> ! {
     // for use in embedded drivers). Channels for EH usage are included: `stm32hal2::adc::AdcChannel::C3`
     let reading = adc.read(chan_num);
 
+    // Convert a reading to voltage, which includes compensation for the built-in VDDA
+    // reference voltage
+    let voltage = adc.reading_to_voltage(reading);
+
     // Or, start reading in continuous mode, reading a single channel
     adc.start_conversion(&[chan_num], OperationMode::Continuous);
 
