@@ -16,17 +16,14 @@ use cortex_m_rt::entry;
 
 use stm32_hal2::{
     clocks::Clocks,
-    delay::Delay,
     dma::{self, Dma, DmaChannel, DmaInterrupt, DmaWriteBuf},
-    gpio::{Edge, PinMode, PinNum},
+    gpio::{Edge, PinMode},
     low_power, pac,
-    spi::{self, Spi, SpiConfig, SpiDevice},
+    prelude::*,
+    spi::{self, BaudRate, Spi, SpiConfig, SpiDevice, SpiMode},
 };
 
-use embedded_hal::{
-    blocking::spi::{Transfer, Write},
-    spi::{Mode, Phase, Polarity},
-};
+make_globals!((SPI, Spi<SPI1>), (DMA, Dma<DMA1>),);
 
 #[entry]
 fn main() -> ! {
@@ -37,36 +34,35 @@ fn main() -> ! {
 
     let clock_cfg = Clocks::default();
 
-    if clock_cfg.setup(&mut dp.RCC, &mut dp.FLASH).is_err() {
-        defmt::error!("Unable to configure clocks due to a speed error.")
-    };
+    clock_cfg.setup(&mut dp.RCC, &mut dp.FLASH).unwrap();
 
     // Enable the GPIOB port.
-    let mut gpioa = GpioA::new(dp.GPIOB, &mut dp.RCC);
+    let mut gpioa = GpioA::new(dp.GPIOB);
 
-    // Configure pins for I2c.
-    let _sck = gpioa.new_pin(PinNum::P5, PinMode::Alt(AltFn::Af5));
-    let _miso = gpioa.new_pin(PinNum::P6, PinMode::Alt(AltFn::Af5));
-    let _mosi = gpioa.new_pin(PinNum::P7, PinMode::Alt(AltFn::Af5));
+    // Configure pins for Spi
+    let _sck = gpioa.new_pin(5, PinMode::Alt(5));
+    let _miso = gpioa.new_pin(6, PinMode::Alt(5));
+    let _mosi = gpioa.new_pin(7, PinMode::Alt(5));
 
-    let cs = gpioa.new_pin(PinNum::P1, PinMode::Output);
+    let cs = gpioa.new_pin(1, PinMode::Output);
 
     let spi_cfg = SpiConfig {
-        mode: Mode {
-            polarity: Polarity::IdleLow,
-            phase: Phase::CaptureOnFirstTransition,
-        },
+        mode: SpiMode::mode1(),
+        // `SpiConfig::default` is mode 0, full duplex, with software CS.
         ..Default::default()
     };
+    // Alternatively, we can configure Mode py polarity and phase:
+    // mode: SpiMode {
+    //     polarity: SpiPolarity::IdleLow,
+    //     phase: SpiPhase::CaptureOnFirstTransition,
+    // }
 
     // Set up an SPI peripheral, running at 4Mhz, in SPI mode 0.
     let spi = Spi::new(
         dp.SPI1,
         SpiDevice::One,
         spi_cfg,
-        4_000_000,
-        &clock_cfg,
-        &mut dp.RCC,
+        BaudRate::Div32, // Eg 80Mhz apb clock / 32 = 2.5Mhz SPI clock.
     );
 
     // Set up DMA, for nonblocking (generally faster) conversion transfers:
@@ -80,8 +76,10 @@ fn main() -> ! {
 
     cs.set_low();
 
-    spi.write_dma(&read_buf, DmaChannel::C3, &mut dma);
-    spi.read_dma(&mut read_buf, DmaChannel::C2, &mut dma);
+    unsafe {
+        spi.write_dma(&read_buf, DmaChannel::C3, &mut dma);
+        spi.read_dma(&mut read_buf, DmaChannel::C2, &mut dma);
+    }
 
     while !dma.transfer_is_complete(DmaChannel::C2) {}
     spi.stop_dma(DmaChannel::C2, &mut dma);
@@ -91,10 +89,17 @@ fn main() -> ! {
 
     defmt::info!("Data: {}", read_buf);
 
-    // Alternatively, use the blocking, non-DMA I2C API provided by `embedded-hal`:
+    // Alternatively, use the blocking, non-DMA SPI API` (Also supports `embedded-hal` traits):
+    spi.write(&write_buf).ok();
     spi.transfer(&mut read_buf).ok();
     defmt::info!("Data: {}", read_buf);
 
+    // Assign peripheral structs as global, so we can access them in interrupts.
+    free(|cs| {
+        DMA.borrow(cs).replace(Some(dma));
+        SPI.borrow(cs).replace(Some(spi));
+    });
+
     // Unmask the interrupt line. See the `DMA_CH2` and `DMA_CH3` interrupt handlers below.
     unsafe {
         NVIC::unmask(pac::Interrupt::DMA1_CH2);
