@@ -12,10 +12,9 @@ use cortex_m_rt::entry;
 
 use stm32_hal2::{
     clocks::Clocks,
-    delay::Delay,
-    gpio::{Edge, PinMode, PinNum},
+    gpio::{Edge, PinMode},
     low_power, pac,
-    timer::{Channel, CountDir, OutputCompare, Timer, TimerInterrupt},
+    timer::{CountDir, OutputCompare, TimChannel, Timer, TimerInterrupt},
 };
 
 #[entry]
@@ -27,25 +26,26 @@ fn main() -> ! {
 
     let clock_cfg = Clocks::default();
 
-    if clock_cfg.setup(&mut dp.RCC, &mut dp.FLASH).is_err() {
-        defmt::error!("Unable to configure clocks due to a speed error.")
-    };
+    clock_cfg.setup(&mut dp.RCC, &mut dp.FLASH).unwrap();
 
-    // Enable the GPIOB port.
-    let mut gpioa = GpioA::new(dp.GPIOB, &mut dp.RCC);
+    // Enable the GPIOA port.
+    let mut gpioa = GpioA::new(dp.GPIOA);
 
     // Set up a PWM pin
-    let _pwm_pin = gpioa.new_pin(PinNum::P0, PinMode::Alt(AltFn::Af1));
+    let _pwm_pin = gpioa.new_pin(0, PinMode::Alt(1));
 
     // Set up a PWM timer that will output to PA0, run at 2400Hz in edge-aligned mode,
     // count up, with a 50% duty cycle.
-    let mut pwm_timer = Timer::new_tim2(dp.TIM2, 2_400., &clock_cfg, &mut dp.RCC);
-    pwm_timer.enable_pwm_output(Channel::One, OutputCompare::Pwm1, CountDir::Up, 0.5);
+    let mut pwm_timer = Timer::new_tim2(dp.TIM2, 2_400., &clock_cfg);
+    pwm_timer.enable_pwm_output(TimChannel::C1, OutputCompare::Pwm1, CountDir::Up, 0.5);
     // Setting auto reload preload allow changing frequency (period) while the timer is running.
     pwm_timer.set_auto_reload_preload(true);
     pwm_timer.enable();
 
-    let mut countdown_timer = Timer::new_tim3(dp.TIM3, 0.5, &clock_cfg, &mut dp.RCC);
+    // Change the duty cycle.
+    pwm_timer.set_duty(TimChannel::C1, 100);
+
+    let mut countdown_timer = Timer::new_tim3(dp.TIM3, 0.5, &clock_cfg);
     countdown_timer.enable_interrupt(TimerInterrupt::Update); // Enable update event interrupts.
     countdown_timer.enable();
 
@@ -73,6 +73,8 @@ fn TIM3() {
     free(|cs| {
         // Clear the interrupt flag. If you ommit this, it will fire repeatedly.
         unsafe { (*pac::TIM3::ptr()).sr.modify(|_, w| w.uif().set_bit()) }
+        // If you have access to the timer variable, eg through a Mutex, you can do this instead:
+        // countdown_timer_clear_interrupt(TimerInterrupt::Update);
 
         defmt::info!("Countdown expired");
     });
