@@ -11,7 +11,7 @@ use cortex_m_rt::entry;
 
 use stm32_hal2::{
     clocks::{Clocks, CrsSyncSr},
-    gpio::{AltFn, GpioA, PinNum},
+    gpio::{AltFn, GpioA, PinMode},
     pac,
     usb::{Peripheral, UsbBus, UsbBusType},
 };
@@ -32,9 +32,7 @@ fn main() -> ! {
     clock_cfg.hsi48_on = true;
     clock_cfg.clk48_src = Clk48Src::Hsi48;
 
-    if clock_cfg.setup(&mut dp.RCC, &mut dp.FLASH).is_err() {
-        defmt::error!("Unable to configure clocks due to a speed error.")
-    };
+    clock_cfg.setup(&mut dp.RCC, &mut dp.FLASH).unwrap();
 
     // Enable the Clock Recovery System, which improves HSI48 accuracy.
     clocks::enable_crs(CrsSyncSrc::Usb, &mut dp.CRS, &mut dp.RCC);
@@ -49,8 +47,8 @@ fn main() -> ! {
     let mut gpioa = GpioA::new(dp.GPIOA, &mut dp.RCC);
 
     // Set up USB pins.
-    let _usb_dm = gpioa.new_pin(PinNum::P11, PinMode::Alt(AltFn::Af14));
-    let _usb_dp = gpioa.new_pin(PinNum::P12, PinMode::Alt(AltFn::Af14));
+    let _usb_dm = gpioa.new_pin(11, PinMode::Alt(14));
+    let _usb_dp = gpioa.new_pin(12, PinMode::Alt(14));
 
     let usb = Peripheral { usb: dp.USB };
     let usb_bus = UsbBus::new(usb);
