@@ -1,14 +1,16 @@
-//! API for the ADC (Analog to Digital Converter)
+//! Provides functionality for the ADC (Analog to Digital Converter).
 
-use cortex_m::asm;
+use cortex_m::{asm, interrupt::free};
+
+#[cfg(feature = "embedded-hal")]
 use embedded_hal::adc::{Channel, OneShot};
 
-use core::ops::Deref;
+use core::{ops::Deref, ptr};
 
 use crate::{
+    clocks::Clocks,
     pac::{self, RCC},
     rcc_en_reset,
-    traits::ClockCfg,
 };
 
 use cfg_if::cfg_if;
@@ -51,7 +53,7 @@ pub mod AdcChannel {
     pub struct C20;
 }
 
-#[derive(Clone, Copy)]
+#[derive(Clone, Copy, PartialEq)]
 pub enum AdcDevice {
     One,
     Two,
@@ -78,18 +80,6 @@ pub enum AdcInterrupt {
     InjectedOverflow,
 }
 
-/// Represents an Analog to Digital Converter (ADC) peripheral.
-pub struct Adc<ADC> {
-    /// ADC Register
-    regs: ADC,
-    device: AdcDevice,
-    ckmode: ClockMode,
-    operation_mode: OperationMode,
-    // Most families use u8 values for calibration, but H7 uses u16.
-    cal_single_ended: Option<u16>, // Stored calibration value for single-ended
-    cal_differential: Option<u16>, // Stored calibration value for differential
-}
-
 // todo: Adc sampling time below depends on the STM32 family. Eg the numbers below
 // todo are wrong for L4, but the idea is the same.
 /// ADC sampling time
@@ -137,7 +127,6 @@ pub enum InputType {
 #[derive(Clone, Copy)]
 #[repr(u8)]
 /// ADC operation mode
-// TODO: Implement other modes
 pub enum OperationMode {
     /// OneShot Mode
     OneShot = 0,
@@ -229,6 +218,21 @@ impl Default for Align {
     }
 }
 
+/// Represents an Analog to Digital Converter (ADC) peripheral.
+pub struct Adc<R> {
+    /// ADC Register
+    regs: R,
+    // Note: We don't own the common regs; pass them mutably where required, since they may be used
+    // by a different ADC.
+    device: AdcDevice,
+    ckmode: ClockMode,
+    operation_mode: OperationMode,
+    // Most families use u8 values for calibration, but H7 uses u16.
+    cal_single_ended: Option<u16>, // Stored calibration value for single-ended
+    cal_differential: Option<u16>, // Stored calibration value for differential
+    vdda_calibrated: f32,
+}
+
 // Abstract implementation of ADC functionality
 macro_rules! hal {
     ($ADC:ident, $ADC_COMMON:ident, $adc:ident, $rcc_num:tt) => {
@@ -243,13 +247,12 @@ macro_rules! hal {
                 /// * the clocksetting is not well defined.
                 /// * the clock was already enabled with a different setting
                 ///
-                pub fn [<new_ $adc>]<C: ClockCfg>(
+                pub fn [<new_ $adc>](
                     regs: pac::$ADC,
                     device: AdcDevice,
-                    adc_common : &mut pac::$ADC_COMMON,
+                    common_regs : &mut pac::$ADC_COMMON,
                     ckmode: ClockMode,
-                    clocks: &C,
-                    rcc: &mut RCC,
+                    clocks: &Clocks,
                 ) -> Self {
                     let mut result = Self {
                         regs,
@@ -258,9 +261,10 @@ macro_rules! hal {
                         operation_mode: OperationMode::OneShot,
                         cal_single_ended: None,
                         cal_differential: None,
+                        vdda_calibrated: 0.
                     };
 
-                    result.enable_clock(adc_common, rcc);
+                    result.enable_clock(common_regs);
                     result.set_align(Align::default());
 
                     result.advregen_enable(clocks);
@@ -282,29 +286,38 @@ macro_rules! hal {
 
                     result.setup_oneshot(); // todo: Setup Continuous
 
+                    // Set up VDDA only after the ADC is otherwise enabled.
+                    result.setup_vdda(common_regs, clocks);
+
                     result
                 }
             }
 
             /// Enable the ADC clock, and set the clock mode.
-            fn enable_clock(&self, common_regs: &mut pac::$ADC_COMMON, rcc: &mut RCC) {
-                paste! {
-                    cfg_if! {
-                        if #[cfg(any(feature = "f3", feature = "h7"))] {
-                            rcc_en_reset!(ahb1, [<adc $rcc_num>], rcc);
-                        } else if #[cfg(feature = "f4")] {
-                            rcc_en_reset!(2, [<adc $rcc_num>], rcc);
-                        } else if #[cfg(any(feature = "h7"))] {
-                        // todo: 1 and 2 are on ahb1enr etc. 3 is on ahb4. 3 won't work here.
-                            rcc_en_reset!(ahb1, [<adc $rcc_num>], rcc);
-                        } else if #[cfg(any(feature = "g4"))] {
-                            rcc_en_reset!(ahb2, [<adc $rcc_num>], rcc);
-                        } else {  // ie L4, L5, G0(?)
-                            rcc_en_reset!(ahb2, adc, rcc);
+            fn enable_clock(&self, regs_common: &mut pac::$ADC_COMMON) {
+            // todo: Consider merging this code into `new`, as we do in other modules.
+                free(|cs| {
+                    let mut rcc = unsafe { &(*RCC::ptr()) };
+
+                    paste! {
+                        cfg_if! {
+                            if #[cfg(any(feature = "f3", feature = "h7"))] {
+                                rcc_en_reset!(ahb1, [<adc $rcc_num>], rcc);
+                            } else if #[cfg(feature = "f4")] {
+                                rcc_en_reset!(2, [<adc $rcc_num>], rcc);
+                            } else if #[cfg(any(feature = "h7"))] {
+                            // todo: 1 and 2 are on ahb1enr etc. 3 is on ahb4. 3 won't work here.
+                                rcc_en_reset!(ahb1, [<adc $rcc_num>], rcc);
+                            } else if #[cfg(any(feature = "g4"))] {
+                                rcc_en_reset!(ahb2, [<adc $rcc_num>], rcc);
+                            } else {  // ie L4, L5, G0(?)
+                                rcc_en_reset!(ahb2, adc, rcc);
+                            }
                         }
                     }
-                }
-                common_regs.ccr.modify(|_, w| unsafe { w.ckmode().bits(self.ckmode as u8) });
+                });
+
+                regs_common.ccr.modify(|_, w| unsafe { w.ckmode().bits(self.ckmode as u8) });
 
             }
 
@@ -335,14 +348,7 @@ macro_rules! hal {
                     panic!("ADC sequence length must be in 1..=16")
                 }
 
-                // self.regs.sqr1.modify(|_, w| unsafe { w.l().bits(len - 1) });
-                cfg_if! {
-                    if #[cfg(any(feature = "l4x1", feature = "l4x2", feature = "l4x3", feature = "l4x5"))] {
-                        self.regs.sqr1.modify(|_, w| unsafe { w.l3().bits(len - 1) });
-                    } else {
-                        self.regs.sqr1.modify(|_, w| unsafe { w.l().bits(len - 1) });
-                    }
-                }
+                self.regs.sqr1.modify(|_, w| unsafe { w.l().bits(len - 1) });
             }
 
             pub fn set_align(&self, align: Align) {
@@ -425,7 +431,7 @@ macro_rules! hal {
             }
 
             /// Enable the voltage regulator, and exit deep sleep mode (some MCUs)
-            pub fn advregen_enable<C: ClockCfg>(&mut self, clocks: &C){
+            pub fn advregen_enable(&mut self, clocks: &Clocks){
                 cfg_if! {
                     if #[cfg(feature = "f3")] {
                         // `F303 RM, 15.3.6:
@@ -477,7 +483,7 @@ macro_rules! hal {
             /// Wait for the advregen to startup.
             ///
             /// This is based on the MAX_ADVREGEN_STARTUP_US of the device.
-            fn wait_advregen_startup<C: ClockCfg>(&self, clocks: &C) {
+            fn wait_advregen_startup(&self, clocks: &Clocks) {
                 let mut delay = (MAX_ADVREGEN_STARTUP_US * 1_000_000) / clocks.sysclk();
                 // https://github.com/rust-embedded/cortex-m/pull/328
                 if delay < 2 {  // Work around a bug in cortex-m.
@@ -489,7 +495,7 @@ macro_rules! hal {
             /// Calibrate. See L4 RM, 16.5.8, or F404 RM, section 15.3.8.
             /// Stores calibration values, which can be re-inserted later,
             /// eg after entering ADC deep sleep mode, or MCU STANDBY or VBAT.
-            pub fn calibrate<C: ClockCfg>(&mut self, input_type: InputType, clocks: &C) {
+            pub fn calibrate(&mut self, input_type: InputType, clocks: &Clocks) {
                 // 1. Ensure DEEPPWD=0, ADVREGEN=1 and that ADC voltage regulator startup time has
                 // elapsed.
                 if !self.is_advregen_enabled() {
@@ -631,6 +637,8 @@ macro_rules! hal {
                 self.stop_conversions();
                 unsafe {
                     match chan {
+                        #[cfg(not(feature = "f3"))]
+                        0 => self.regs.smpr1.modify(|_, w| w.smp0().bits(smp as u8)),
                         1 => self.regs.smpr1.modify(|_, w| w.smp1().bits(smp as u8)),
                         2 => self.regs.smpr1.modify(|_, w| w.smp2().bits(smp as u8)),
                         3 => self.regs.smpr1.modify(|_, w| w.smp3().bits(smp as u8)),
@@ -655,10 +663,133 @@ macro_rules! hal {
                 }
             }
 
+            /// Set up the internal voltage reference, to improve conversion from reading
+            /// to voltage accuracy. See L44 RM, section 16.4.34: "Monitoring the internal voltage reference"
+            fn setup_vdda(&mut self, regs_common: &mut pac::$ADC_COMMON, clocks: &Clocks) {
+                // todo: Using ints intead of floats here and in read_voltage, for use on Cortex M0s?
+                // todo: May need to use mv.
+                regs_common.ccr.modify(|_, w| w.vrefen().set_bit());
+                // "Table 24. Embedded internal voltage reference" states that it takes a maximum of 12 us
+                // to stabilize the internal voltage reference, we wait a little more.
+                let mut delay = (15_000_000) / clocks.sysclk();
+                // https://github.com/rust-embedded/cortex-m/pull/328
+                if delay < 2 {  // Work around a bug in cortex-m.
+                    delay = 2;
+                }
+                asm::delay(delay);
+
+
+                // (From L4xx-hal) "Table 24. Embedded internal voltage reference" states that the sample time needs to be
+                // at a minimum 4 us. With 640.5 ADC cycles we have a minimum of 8 us at 80 MHz, leaving
+                // some headroom.
+                // todo: Reset existing sample time after this measurement?
+                // todo: This may not work properly on variants clocked faster than 80Mhz(?)
+
+                // RM: It is possible to monitor the internal voltage reference (VREFINT) to have a reference point for
+                // evaluating the ADC VREF+ voltage level.
+                // The internal voltage reference is internally connected to the input channel 0 of the ADC1
+                // (ADC1_INP0).
+
+                // Regardless of which ADC we're on, we take this reading using ADC1.
+                #[cfg(not(any(feature = "l5", feature = "g4")))]
+                let vref_reading = if self.device != AdcDevice::One {
+                    // todo: What if ADC1 is alreayd enabled and configured differently?
+                    // todo: Either way, if you're also using ADC1, this will screw things up⋅.
+
+                    // If we're currently using ADC1 (and this is a different ADC), skip this step for now;
+                    // VDDA will be wrong,
+                    // and all readings using voltage conversion will be wrong.
+                    if unsafe { (*pac::ADC1::ptr()).cr.read().aden().bit_is_set() } {
+                        // todo: Take an ADC1 reading if this is the case.
+                        return
+                    }
+
+                    let mut dp = unsafe { pac::Peripherals::steal() };
+
+                    cfg_if! {
+                        if #[cfg(feature = "f3")] {
+                            let mut common_regs = dp.ADC1_2;
+                        } else if #[cfg(any(feature = "l4", feature = "l5"))] {
+                            let mut common_regs = dp.ADC_COMMON;
+                        } else {
+                            let mut common_regs = dp.ADC12_COMMON;
+                        }
+                    }
+
+                    let mut adc1 = Adc::new_adc1(
+                        dp.ADC1,
+                        AdcDevice::One,
+                        &mut common_regs,
+                        ClockMode::default(),
+                        clocks,
+                    );
+                    adc1.set_sample_time(0, SampleTime::T601);
+                    let reading = adc1.read(0);
+
+                    // Disable ADC1 and its clock.
+                    adc1.disable();
+                    // todo: Disable the ADC1 clock. (Note it's on 3 diff regs depending on family)
+
+                    reading
+                } else {
+                    self.set_sample_time(0, SampleTime::T601);
+                    self.read(0)
+                };
+
+                #[cfg(feature = "l5")]
+                let vref_reading = 0.; // todo handle this! Just take a reading off the only ADC.
+
+                #[cfg(feature = "g4")]
+                let vref_reading = 0.; // todo handle this! Weird macro issue (chicken+egg?)
+
+                // self.set_sample_time(0, old_sample_time);
+                regs_common.ccr.modify(|_, w| w.vrefen().clear_bit());
+
+
+                // The VDDA power supply voltage applied to the microcontroller may be subject to variation or
+                // not precisely known. The embedded internal voltage reference (VREFINT) and its calibration
+                // data acquired by the ADC during the manufacturing process at VDDA = 3.0 V can be used to
+                // evaluate the actual VDDA voltage level.
+                // The following formula gives the actual VDDA voltage supplying the device:
+                // VDDA = 3.0 V x VREFINT_CAL / VREFINT_DATA
+                // where:
+                // • VREFINT_CAL is the VREFINT calibration value
+                // • VREFINT_DATA is the actual VREFINT output value converted by ADC
+
+                // todo: This address may be different on different MCUs, even within the same family!!
+                let vrefint_cal: u16 = unsafe { ptr::read_volatile(&*(0x1FFF_75AA as *const _)) };
+
+                self.vdda_calibrated = 3. * vrefint_cal as f32 / vref_reading as f32
+            }
+
+            /// Convert a raw measurement into a voltage in Volts, using the calibrated VDDA.
+            /// See RM0394, section 16.4.34
+            pub fn reading_to_voltage(&self, reading: u16) -> f32 {
+                // RM:
+                // Converting a supply-relative ADC measurement to an absolute voltage value
+                // The ADC is designed to deliver a digital value corresponding to the ratio between the analog
+                // power supply and the voltage applied on the converted channel. For most application use
+                // cases, it is necessary to convert this ratio into a voltage independent of VDDA. For
+                // applications where VDDA is known and ADC converted values are right-aligned you can use
+                // the following formula to get this absolute value:
+
+                // V_CHANNELx = V_DDA / FULL_SCALE x ADCx_DATA
+
+                // Where:
+                // • VREFINT_CAL is the VREFINT calibration value
+                // • ADC_DATA is the value measured by the ADC on channel x (right-aligned)
+                // • VREFINT_DATA is the actual VREFINT output value converted by the ADC
+                // • FULL_SCALE is the maximum digital value of the ADC output. For example with 12-bit
+                // resolution, it will be 212 − 1 = 4095 or with 8-bit resolution, 28 − 1 = 255
+                // todo: Pass vdda here, or to teh struct?
+                // todo: FULL_SCALE will be different for 16-bit. And differential?
+                self.vdda_calibrated / 4_096. * reading as f32
+            }
+
             /// Start a conversion: Either a single measurement, or continuous conversions.
             /// See L4 RM 16.4.15 for details.
             pub fn start_conversion(&mut self, sequence: &[u8], mode: OperationMode) {
-                // Set continuous or differential mode.
+                // Set continuous or one-shot mode.
                 self.regs.cfgr.modify(|_, w| w.cont().bit(mode as u8 != 0));
                 // todo: You should call this elsewhere, once, to prevent unneded reg writes.
                 for (i, channel) in sequence.iter().enumerate() {
@@ -693,10 +824,12 @@ macro_rules! hal {
                 self.read_result()
             }
 
+            // todo: fn read_voltage, using vrefint and L4xx-hal style calibration?
+
             #[cfg(not(any(feature = "g0", feature = "h7", feature = "f4", feature = "l5")))]
             /// Take a one shot reading, using DMA. See L44 RM, 16.4.27: "DMA one shot mode".
             /// Note that the `channel` argument is only used on F3 and L4.
-            pub fn read_dma<D>(&mut self, buf: &mut [u16], adc_channel: u8, dma_channel: DmaChannel, dma: &mut Dma<D>)
+            pub unsafe fn read_dma<D>(&mut self, buf: &mut [u16], adc_channel: u8, dma_channel: DmaChannel, dma: &mut Dma<D>)
             where
                 D: Deref<Target = dma_p::RegisterBlock>,
             {
@@ -783,37 +916,37 @@ macro_rules! hal {
 
             /// Enable a specific type of ADC interrupt.
             pub fn enable_interrupt(&mut self, interrupt: AdcInterrupt) {
-                match interrupt {
-                    AdcInterrupt::Ready => self.regs.ier.modify(|_, w| w.adrdyie().set_bit()),
-                    AdcInterrupt::EndOfConversion => self.regs.ier.modify(|_, w| w.eocie().set_bit()),
-                    AdcInterrupt::EnfOfSequence => self.regs.ier.modify(|_, w| w.eosie().set_bit()),
-                    AdcInterrupt::EndofConversionInjected => self.regs.ier.modify(|_, w| w.jeocie().set_bit()),
-                    AdcInterrupt::EndOfSequenceInjected => self.regs.ier.modify(|_, w| w.jeosie().set_bit()),
-                    AdcInterrupt::Watchdog1 => self.regs.ier.modify(|_, w| w.awd1ie().set_bit()),
-                    AdcInterrupt::Watchdog2 => self.regs.ier.modify(|_, w| w.awd2ie().set_bit()),
-                    AdcInterrupt::Watchdog3 => self.regs.ier.modify(|_, w| w.awd3ie().set_bit()),
-                    AdcInterrupt::EndOfSamplingPhase => self.regs.ier.modify(|_, w| w.eosmpie().set_bit()),
-                    AdcInterrupt::Overrun => self.regs.ier.modify(|_, w| w.ovrie().set_bit()),
-                    AdcInterrupt::InjectedOverflow => self.regs.ier.modify(|_, w| w.jqovfie().set_bit()),
-                }
+                self.regs.ier.modify(|_, w| match interrupt {
+                    AdcInterrupt::Ready => w.adrdyie().set_bit(),
+                    AdcInterrupt::EndOfConversion => w.eocie().set_bit(),
+                    AdcInterrupt::EnfOfSequence => w.eosie().set_bit(),
+                    AdcInterrupt::EndofConversionInjected => w.jeocie().set_bit(),
+                    AdcInterrupt::EndOfSequenceInjected => w.jeosie().set_bit(),
+                    AdcInterrupt::Watchdog1 => w.awd1ie().set_bit(),
+                    AdcInterrupt::Watchdog2 => w.awd2ie().set_bit(),
+                    AdcInterrupt::Watchdog3 => w.awd3ie().set_bit(),
+                    AdcInterrupt::EndOfSamplingPhase => w.eosmpie().set_bit(),
+                    AdcInterrupt::Overrun => w.ovrie().set_bit(),
+                    AdcInterrupt::InjectedOverflow => w.jqovfie().set_bit(),
+                });
             }
 
             /// Clear an interrupt flag of the specified type. Consider running this in the
             /// corresponding ISR.
             pub fn clear_interrupt(&mut self, interrupt: AdcInterrupt) {
-                match interrupt {
-                    AdcInterrupt::Ready => self.regs.isr.modify(|_, w| w.adrdy().set_bit()),
-                    AdcInterrupt::EndOfConversion => self.regs.isr.modify(|_, w| w.eoc().set_bit()),
-                    AdcInterrupt::EnfOfSequence => self.regs.isr.modify(|_, w| w.eos().set_bit()),
-                    AdcInterrupt::EndofConversionInjected => self.regs.isr.modify(|_, w| w.jeoc().set_bit()),
-                    AdcInterrupt::EndOfSequenceInjected => self.regs.isr.modify(|_, w| w.jeos().set_bit()),
-                    AdcInterrupt::Watchdog1 => self.regs.isr.modify(|_, w| w.awd1().set_bit()),
-                    AdcInterrupt::Watchdog2 => self.regs.isr.modify(|_, w| w.awd2().set_bit()),
-                    AdcInterrupt::Watchdog3 => self.regs.isr.modify(|_, w| w.awd3().set_bit()),
-                    AdcInterrupt::EndOfSamplingPhase => self.regs.isr.modify(|_, w| w.eosmp().set_bit()),
-                    AdcInterrupt::Overrun => self.regs.isr.modify(|_, w| w.ovr().set_bit()),
-                    AdcInterrupt::InjectedOverflow => self.regs.isr.modify(|_, w| w.jqovf().set_bit()),
-                }
+                self.regs.isr.modify(|_, w| match interrupt {
+                    AdcInterrupt::Ready =>  w.adrdy().set_bit(),
+                    AdcInterrupt::EndOfConversion =>  w.eoc().set_bit(),
+                    AdcInterrupt::EnfOfSequence =>  w.eos().set_bit(),
+                    AdcInterrupt::EndofConversionInjected =>  w.jeoc().set_bit(),
+                    AdcInterrupt::EndOfSequenceInjected =>  w.jeos().set_bit(),
+                    AdcInterrupt::Watchdog1 =>  w.awd1().set_bit(),
+                    AdcInterrupt::Watchdog2 =>  w.awd2().set_bit(),
+                    AdcInterrupt::Watchdog3 =>  w.awd3().set_bit(),
+                    AdcInterrupt::EndOfSamplingPhase =>  w.eosmp().set_bit(),
+                    AdcInterrupt::Overrun =>  w.ovr().set_bit(),
+                    AdcInterrupt::InjectedOverflow =>  w.jqovf().set_bit(),
+                });
                 // match interrupt {
                 //     AdcInterrupt::Ready => self.regs.icr.write(|_w| w.adrdy().set_bit()),
                 //     AdcInterrupt::EndOfConversion => self.regs.icr.write(|w| w.eoc().set_bit()),
@@ -830,6 +963,8 @@ macro_rules! hal {
             }
         }
 
+        #[cfg(feature = "embedded-hal")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
         impl<WORD, PIN> OneShot<pac::$ADC, WORD, PIN> for Adc<pac::$ADC>
         where
             WORD: From<u16>,
@@ -842,83 +977,123 @@ macro_rules! hal {
                 }
         }
 
+        #[cfg(feature = "embedded-hal")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
         // todo: This mess is due to how EH implements the OneShot trait.
         impl Channel<pac::$ADC> for AdcChannel::C1 {
             type ID = u8;
             fn channel() -> u8 { 1 }
         }
+        #[cfg(feature = "embedded-hal")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
         impl Channel<pac::$ADC> for AdcChannel::C2 {
             type ID = u8;
             fn channel() -> u8 { 2 }
         }
+        #[cfg(feature = "embedded-hal")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
         impl Channel<pac::$ADC> for AdcChannel::C3 {
             type ID = u8;
             fn channel() -> u8 { 3}
         }
+        #[cfg(feature = "embedded-hal")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
         impl Channel<pac::$ADC> for AdcChannel::C4 {
             type ID = u8;
             fn channel() -> u8 { 4 }
         }
+        #[cfg(feature = "embedded-hal")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
         impl Channel<pac::$ADC> for AdcChannel::C5 {
             type ID = u8;
             fn channel() -> u8 { 5 }
         }
+        #[cfg(feature = "embedded-hal")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
         impl Channel<pac::$ADC> for AdcChannel::C6 {
             type ID = u8;
             fn channel() -> u8 { 6 }
         }
+        #[cfg(feature = "embedded-hal")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
         impl Channel<pac::$ADC> for AdcChannel::C7 {
             type ID = u8;
             fn channel() -> u8 { 7 }
         }
+        #[cfg(feature = "embedded-hal")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
         impl Channel<pac::$ADC> for AdcChannel::C8 {
             type ID = u8;
             fn channel() -> u8 { 8 }
         }
+        #[cfg(feature = "embedded-hal")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
         impl Channel<pac::$ADC> for AdcChannel::C9 {
             type ID = u8;
             fn channel() -> u8 { 9 }
         }
+        #[cfg(feature = "embedded-hal")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
         impl Channel<pac::$ADC> for AdcChannel::C10 {
             type ID = u8;
             fn channel() -> u8 { 10 }
         }
+        #[cfg(feature = "embedded-hal")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
         impl Channel<pac::$ADC> for AdcChannel::C11 {
             type ID = u8;
             fn channel() -> u8 { 11 }
         }
+        #[cfg(feature = "embedded-hal")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
         impl Channel<pac::$ADC> for AdcChannel::C12 {
             type ID = u8;
             fn channel() -> u8 { 12 }
         }
+        #[cfg(feature = "embedded-hal")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
         impl Channel<pac::$ADC> for AdcChannel::C13 {
             type ID = u8;
             fn channel() -> u8 { 13 }
         }
+        #[cfg(feature = "embedded-hal")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
         impl Channel<pac::$ADC> for AdcChannel::C14 {
             type ID = u8;
             fn channel() -> u8 { 14 }
         }
+        #[cfg(feature = "embedded-hal")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
         impl Channel<pac::$ADC> for AdcChannel::C15 {
             type ID = u8;
             fn channel() -> u8 { 15 }
         }
+        #[cfg(feature = "embedded-hal")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
         impl Channel<pac::$ADC> for AdcChannel::C16 {
             type ID = u8;
             fn channel() -> u8 { 16 }
         }
+        #[cfg(feature = "embedded-hal")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
         impl Channel<pac::$ADC> for AdcChannel::C17 {
             type ID = u8;
             fn channel() -> u8 { 17 }
         }
+        #[cfg(feature = "embedded-hal")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
         impl Channel<pac::$ADC> for AdcChannel::C18 {
             type ID = u8;
             fn channel() -> u8 { 18 }
         }
+        #[cfg(feature = "embedded-hal")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
         impl Channel<pac::$ADC> for AdcChannel::C19 {
             type ID = u8;
             fn channel() -> u8 { 19 }
         }
+        #[cfg(feature = "embedded-hal")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
         impl Channel<pac::$ADC> for AdcChannel::C20 {
             type ID = u8;
             fn channel() -> u8 { 20 }
@@ -941,7 +1116,13 @@ hal!(ADC4, ADC3_4, adc4, 34);
 #[cfg(any(feature = "l4"))]
 hal!(ADC1, ADC_COMMON, adc1, _);
 
-#[cfg(any(feature = "l4x1", feature = "l4x2", feature = "l4x5", feature = "l4x6",))]
+#[cfg(any(
+    feature = "l4x1",
+    feature = "l4x2",
+    feature = "l412",
+    feature = "l4x5",
+    feature = "l4x6",
+))]
 hal!(ADC2, ADC_COMMON, adc2, _);
 
 #[cfg(any(feature = "l4x5", feature = "l4x6",))]
