@@ -11,7 +11,7 @@ cfg_if::cfg_if! {
     if #[cfg(any(feature = "f3", feature = "f4"))] {
         mod f;
         pub use f::*;
-    } else if #[cfg(any(feature = "l4", feature = "l5", feature = "g0", feature = "g4", feature = "wb"))] {
+    } else if #[cfg(any(feature = "l4", feature = "l5", feature = "g0", feature = "g4", feature = "wb", feature = "wl"))] {
         mod baseline;
         pub use baseline::*;
     } else if #[cfg(feature = "u5")] {
@@ -32,3 +32,10 @@ cfg_if::cfg_if! {
 /// Speed out of limits.
 #[derive(Debug)]
 pub struct SpeedError {}
+
+#[derive(Clone, Copy)]
+#[repr(u8)]
+pub enum ClocksValid {
+    Valid,
+    NotValid,
+}
