@@ -6,6 +6,8 @@ use core::{
     sync::atomic::{self, Ordering},
 };
 
+use cortex_m::interrupt::free;
+
 use crate::{
     pac::{self, RCC},
     rcc_en_reset,
@@ -16,9 +18,9 @@ use crate::pac::dma;
 #[cfg(not(feature = "g0"))]
 use crate::pac::dma1 as dma;
 
-#[cfg(any(feature = "l5", feature = "g0", feature = "g4"))]
+#[cfg(any(feature = "l5", feature = "g0", feature = "g4", feature = "wl"))]
 use pac::DMAMUX;
-#[cfg(feature = "wb")]
+#[cfg(any(feature = "wb"))]
 use pac::DMAMUX1 as DMAMUX;
 
 // use embedded_dma::{ReadBuffer, WriteBuffer};
@@ -116,7 +118,8 @@ impl DmaInput {
     }
 
     #[cfg(feature = "l4")]
-    /// Find the channel select value for a given DMA input. See L44 RM, Table 41.
+    /// Find the value to set in the DMA_CSELR register, for L4. Ie, channel select value for a given DMA input.
+    /// See L44 RM, Table 41.
     pub fn dma1_channel_select(&self) -> u8 {
         match self {
             Self::Adc1 => 0b000,
@@ -291,11 +294,11 @@ macro_rules! enable_interrupt {
             $ccr.modify(|_, w| w.en().clear_bit());
             while $ccr.read().en().bit_is_set() {}
         }
-        match $interrupt_type {
-            DmaInterrupt::TransferError => $ccr.modify(|_, w| w.teie().set_bit()),
-            DmaInterrupt::HalfTransfer => $ccr.modify(|_, w| w.htie().set_bit()),
-            DmaInterrupt::TransferComplete => $ccr.modify(|_, w| w.tcie().set_bit()),
-        }
+        $ccr.modify(|_, w| match $interrupt_type {
+            DmaInterrupt::TransferError => w.teie().set_bit(),
+            DmaInterrupt::HalfTransfer => w.htie().set_bit(),
+            DmaInterrupt::TransferComplete => w.tcie().set_bit(),
+        });
 
         if originally_enabled {
             $ccr.modify(|_, w| w.en().set_bit());
@@ -334,18 +337,20 @@ impl<D> Dma<D>
 where
     D: Deref<Target = dma::RegisterBlock>,
 {
-    pub fn new(regs: D, rcc: &mut RCC) -> Self {
+    pub fn new(regs: D) -> Self {
         // todo: Enable RCC for DMA 2 etc!
-
-        cfg_if! {
-            if #[cfg(feature = "f3")] {
-                rcc.ahbenr.modify(|_, w| w.dma1en().set_bit()); // no dmarst on F3.
-            } else if #[cfg(feature = "g0")] {
-                rcc_en_reset!(ahb1, dma, rcc);
-            } else {
-                rcc_en_reset!(ahb1, dma1, rcc);
+        free(|cs| {
+            let mut rcc = unsafe { &(*RCC::ptr()) };
+            cfg_if! {
+                if #[cfg(feature = "f3")] {
+                    rcc.ahbenr.modify(|_, w| w.dma1en().set_bit()); // no dmarst on F3.
+                } else if #[cfg(feature = "g0")] {
+                    rcc_en_reset!(ahb1, dma, rcc);
+                } else {
+                    rcc_en_reset!(ahb1, dma1, rcc);
+                }
             }
-        }
+        });
 
         Self { regs }
     }
@@ -537,6 +542,9 @@ where
 
         // 3. Configure the total number of data to transfer in the DMA_CNDTRx register.
         // After each data transfer, this value is decremented.
+        #[cfg(feature = "wl")]
+        let num_data = num_data as u32;
+
         unsafe {
             match channel {
                 DmaChannel::C1 => {
@@ -936,15 +944,15 @@ where
     pub fn channel_select(&mut self, input: DmaInput) {
         // todo: Allow selecting channels in pairs to save a write.
         let val = input.dma1_channel_select();
-        match input.dma1_channel() {
-            DmaChannel::C1 => self.regs.cselr.modify(|_, w| w.c1s().bits(val)),
-            DmaChannel::C2 => self.regs.cselr.modify(|_, w| w.c2s().bits(val)),
-            DmaChannel::C3 => self.regs.cselr.modify(|_, w| w.c3s().bits(val)),
-            DmaChannel::C4 => self.regs.cselr.modify(|_, w| w.c4s().bits(val)),
-            DmaChannel::C5 => self.regs.cselr.modify(|_, w| w.c5s().bits(val)),
-            DmaChannel::C6 => self.regs.cselr.modify(|_, w| w.c6s().bits(val)),
-            DmaChannel::C7 => self.regs.cselr.modify(|_, w| w.c7s().bits(val)),
-        }
+        self.regs.cselr.modify(|_, w| match input.dma1_channel() {
+            DmaChannel::C1 => w.c1s().bits(val),
+            DmaChannel::C2 => w.c2s().bits(val),
+            DmaChannel::C3 => w.c3s().bits(val),
+            DmaChannel::C4 => w.c4s().bits(val),
+            DmaChannel::C5 => w.c5s().bits(val),
+            DmaChannel::C6 => w.c6s().bits(val),
+            DmaChannel::C7 => w.c7s().bits(val),
+        });
     }
 
     /// Enable a specific type of interrupt. Note that the `TransferComplete` interrupt
@@ -1034,210 +1042,108 @@ where
 
     pub fn clear_interrupt(&mut self, channel: DmaChannel, interrupt: DmaInterrupt) {
         cfg_if! {
-            if #[cfg(feature = "g4")] {
-                match channel {
+            if #[cfg(any(feature = "g4", feature = "wl"))] {
+                self.regs.ifcr.write(|w| match channel {
                     DmaChannel::C1 => match interrupt {
-                        DmaInterrupt::TransferError => self.regs.ifcr.write(|w| w.teif1().set_bit()),
-                        DmaInterrupt::HalfTransfer => self.regs.ifcr.write(|w| w.htif1().set_bit()),
-                        DmaInterrupt::TransferComplete => self.regs.ifcr.write(|w| w.tcif1().set_bit()),
+                        DmaInterrupt::TransferError => w.teif1().set_bit(),
+                        DmaInterrupt::HalfTransfer => w.htif1().set_bit(),
+                        DmaInterrupt::TransferComplete => w.tcif1().set_bit(),
                     }
                     DmaChannel::C2 => match interrupt {
-                        DmaInterrupt::TransferError => self.regs.ifcr.write(|w| w.teif2().set_bit()),
-                        DmaInterrupt::HalfTransfer => self.regs.ifcr.write(|w| w.htif2().set_bit()),
-                        DmaInterrupt::TransferComplete => self.regs.ifcr.write(|w| w.tcif2().set_bit()),
+                        DmaInterrupt::TransferError => w.teif2().set_bit(),
+                        DmaInterrupt::HalfTransfer => w.htif2().set_bit(),
+                        DmaInterrupt::TransferComplete => w.tcif2().set_bit(),
                     }
                     DmaChannel::C3 => match interrupt {
-                        DmaInterrupt::TransferError => self.regs.ifcr.write(|w| w.teif3().set_bit()),
-                        DmaInterrupt::HalfTransfer => self.regs.ifcr.write(|w| w.htif3().set_bit()),
-                        DmaInterrupt::TransferComplete => self.regs.ifcr.write(|w| w.tcif3().set_bit()),
+                        DmaInterrupt::TransferError => w.teif3().set_bit(),
+                        DmaInterrupt::HalfTransfer => w.htif3().set_bit(),
+                        DmaInterrupt::TransferComplete => w.tcif3().set_bit(),
                     }
                     DmaChannel::C4 => match interrupt {
-                        DmaInterrupt::TransferError => self.regs.ifcr.write(|w| w.teif4().set_bit()),
-                        DmaInterrupt::HalfTransfer => self.regs.ifcr.write(|w| w.htif4().set_bit()),
-                        DmaInterrupt::TransferComplete => self.regs.ifcr.write(|w| w.tcif4().set_bit()),
+                        DmaInterrupt::TransferError => w.teif4().set_bit(),
+                        DmaInterrupt::HalfTransfer => w.htif4().set_bit(),
+                        DmaInterrupt::TransferComplete => w.tcif4().set_bit(),
                     }
                     DmaChannel::C5 => match interrupt {
-                        DmaInterrupt::TransferError => self.regs.ifcr.write(|w| w.teif5().set_bit()),
-                        DmaInterrupt::HalfTransfer => self.regs.ifcr.write(|w| w.htif5().set_bit()),
-                        DmaInterrupt::TransferComplete => self.regs.ifcr.write(|w| w.tcif5().set_bit()),
+                        DmaInterrupt::TransferError => w.teif5().set_bit(),
+                        DmaInterrupt::HalfTransfer => w.htif5().set_bit(),
+                        DmaInterrupt::TransferComplete => w.tcif5().set_bit(),
                     }
                     DmaChannel::C6 => match interrupt {
-                        DmaInterrupt::TransferError => self.regs.ifcr.write(|w| w.teif6().set_bit()),
-                        DmaInterrupt::HalfTransfer => self.regs.ifcr.write(|w| w.htif6().set_bit()),
-                        DmaInterrupt::TransferComplete => self.regs.ifcr.write(|w| w.tcif6().set_bit()),
+                        DmaInterrupt::TransferError => w.teif6().set_bit(),
+                        DmaInterrupt::HalfTransfer => w.htif6().set_bit(),
+                        DmaInterrupt::TransferComplete => w.tcif6().set_bit(),
                     }
                     DmaChannel::C7 => match interrupt {
-                        DmaInterrupt::TransferError => self.regs.ifcr.write(|w| w.teif7().set_bit()),
-                        DmaInterrupt::HalfTransfer => self.regs.ifcr.write(|w| w.htif7().set_bit()),
-                        DmaInterrupt::TransferComplete => self.regs.ifcr.write(|w| w.tcif7().set_bit()),
+                        DmaInterrupt::TransferError => w.teif7().set_bit(),
+                        DmaInterrupt::HalfTransfer => w.htif7().set_bit(),
+                        DmaInterrupt::TransferComplete => w.tcif7().set_bit(),
                     }
+                    #[cfg(not(feature = "wl"))]
                     DmaChannel::C8 => match interrupt {
-                        DmaInterrupt::TransferError => self.regs.ifcr.write(|w| w.teif8().set_bit()),
-                        DmaInterrupt::HalfTransfer => self.regs.ifcr.write(|w| w.htif8().set_bit()),
-                        DmaInterrupt::TransferComplete => self.regs.ifcr.write(|w| w.tcif8().set_bit()),
+                        DmaInterrupt::TransferError => w.teif8().set_bit(),
+                        DmaInterrupt::HalfTransfer => w.htif8().set_bit(),
+                        DmaInterrupt::TransferComplete => w.tcif8().set_bit(),
                     }
-                }
+                })
             } else {
-                match channel {
+                self.regs.ifcr.write(|w| match channel {
                     DmaChannel::C1 => match interrupt {
-                        DmaInterrupt::TransferError => self.regs.ifcr.write(|w| w.cteif1().set_bit()),
-                        DmaInterrupt::HalfTransfer => self.regs.ifcr.write(|w| w.chtif1().set_bit()),
-                        DmaInterrupt::TransferComplete => self.regs.ifcr.write(|w| w.ctcif1().set_bit()),
+                        DmaInterrupt::TransferError => w.cteif1().set_bit(),
+                        DmaInterrupt::HalfTransfer => w.chtif1().set_bit(),
+                        DmaInterrupt::TransferComplete => w.ctcif1().set_bit(),
                     }
                     DmaChannel::C2 => match interrupt {
-                        DmaInterrupt::TransferError => self.regs.ifcr.write(|w| w.cteif2().set_bit()),
-                        DmaInterrupt::HalfTransfer => self.regs.ifcr.write(|w| w.chtif2().set_bit()),
-                        DmaInterrupt::TransferComplete => self.regs.ifcr.write(|w| w.ctcif2().set_bit()),
+                        DmaInterrupt::TransferError => w.cteif2().set_bit(),
+                        DmaInterrupt::HalfTransfer => w.chtif2().set_bit(),
+                        DmaInterrupt::TransferComplete => w.ctcif2().set_bit(),
                     }
                     DmaChannel::C3 => match interrupt {
-                        DmaInterrupt::TransferError => self.regs.ifcr.write(|w| w.cteif3().set_bit()),
-                        DmaInterrupt::HalfTransfer => self.regs.ifcr.write(|w| w.chtif3().set_bit()),
-                        DmaInterrupt::TransferComplete => self.regs.ifcr.write(|w| w.ctcif3().set_bit()),
+                        DmaInterrupt::TransferError => w.cteif3().set_bit(),
+                        DmaInterrupt::HalfTransfer => w.chtif3().set_bit(),
+                        DmaInterrupt::TransferComplete => w.ctcif3().set_bit(),
                     }
                     DmaChannel::C4 => match interrupt {
-                        DmaInterrupt::TransferError => self.regs.ifcr.write(|w| w.cteif4().set_bit()),
-                        DmaInterrupt::HalfTransfer => self.regs.ifcr.write(|w| w.chtif4().set_bit()),
-                        DmaInterrupt::TransferComplete => self.regs.ifcr.write(|w| w.ctcif4().set_bit()),
+                        DmaInterrupt::TransferError => w.cteif4().set_bit(),
+                        DmaInterrupt::HalfTransfer => w.chtif4().set_bit(),
+                        DmaInterrupt::TransferComplete => w.ctcif4().set_bit(),
                     }
                     DmaChannel::C5 => match interrupt {
-                        DmaInterrupt::TransferError => self.regs.ifcr.write(|w| w.cteif5().set_bit()),
-                        DmaInterrupt::HalfTransfer => self.regs.ifcr.write(|w| w.chtif5().set_bit()),
-                        DmaInterrupt::TransferComplete => self.regs.ifcr.write(|w| w.ctcif5().set_bit()),
+                        DmaInterrupt::TransferError => w.cteif5().set_bit(),
+                        DmaInterrupt::HalfTransfer => w.chtif5().set_bit(),
+                        DmaInterrupt::TransferComplete => w.ctcif5().set_bit(),
                     }
                     #[cfg(not(feature = "g0"))]
                     DmaChannel::C6 => match interrupt {
-                        DmaInterrupt::TransferError => self.regs.ifcr.write(|w| w.cteif6().set_bit()),
-                        DmaInterrupt::HalfTransfer => self.regs.ifcr.write(|w| w.chtif6().set_bit()),
-                        DmaInterrupt::TransferComplete => self.regs.ifcr.write(|w| w.ctcif6().set_bit()),
+                        DmaInterrupt::TransferError => w.cteif6().set_bit(),
+                        DmaInterrupt::HalfTransfer => w.chtif6().set_bit(),
+                        DmaInterrupt::TransferComplete => w.ctcif6().set_bit(),
                     }
                     #[cfg(not(feature = "g0"))]
                     DmaChannel::C7 => match interrupt {
-                        DmaInterrupt::TransferError => self.regs.ifcr.write(|w| w.cteif7().set_bit()),
-                        DmaInterrupt::HalfTransfer => self.regs.ifcr.write(|w| w.chtif7().set_bit()),
-                        DmaInterrupt::TransferComplete => self.regs.ifcr.write(|w| w.ctcif7().set_bit()),
+                        DmaInterrupt::TransferError => w.cteif7().set_bit(),
+                        DmaInterrupt::HalfTransfer => w.chtif7().set_bit(),
+                        DmaInterrupt::TransferComplete => w.ctcif7().set_bit(),
                     }
                     #[cfg(any(feature = "l5", feature = "g4"))]
                     DmaChannel::C8 => match interrupt {
-                        DmaInterrupt::TransferError => self.regs.ifcr.write(|w| w.cteif8().set_bit()),
-                        DmaInterrupt::HalfTransfer => self.regs.ifcr.write(|w| w.chtif8().set_bit()),
-                        DmaInterrupt::TransferComplete => self.regs.ifcr.write(|w| w.ctcif8().set_bit()),
+                        DmaInterrupt::TransferError => w.cteif8().set_bit(),
+                        DmaInterrupt::HalfTransfer => w.chtif8().set_bit(),
+                        DmaInterrupt::TransferComplete => w.ctcif8().set_bit(),
                     }
-                }
+                })
             }
         }
     }
 }
 
-// // todo: Remove the static reqs once you get thi sworking.
-// // todo: If you end up using these, move to util.
-// // todo: Set up a global flag to figure out if this is in use to prevent concurrent SPI
-// // todo activity while in use??
-// // todo: Impl Drop for DmaWriteBuf, where it stops the transfer.
-// pub struct DmaWriteBuf<'a, T> {
-//     // pub buf: &'static [u8]
-//     pub buf: &'a mut [T], // pub channel: DmaChannel,
-//
-//     // #[repr(align(4))]
-//     // struct Aligned<T: ?Sized>(T);
-//     //s tatic mut BUF: Aligned<[u16; 8]> = Aligned([0; 8]);
-// }
-//
-// // unsafe impl StaticWriteBuffer for DmaWriteBuf {
-// //     type Word = u8;
-// //
-// //     unsafe fn static_write_buffer(&mut self) -> (*mut Self::Word, usize) {
-// //         (self.buf.as_mut_ptr(), self.buf.len())
-// //     }
-// // }
-//
-// unsafe impl<'a, T> WriteBuffer for DmaWriteBuf<'a, T> {
-//     type Word = T;
-//
-//     unsafe fn write_buffer(&mut self) -> (*mut Self::Word, usize) {
-//         (self.buf.as_mut_ptr(), self.buf.len())
-//     }
-// }
-//
-// impl<T> Drop for DmaWriteBuf<'_, T> {
-//     // todo: Hardcoded for DMA1 and Chan 3.
-//     // todo: Does this stop all transfers in progress?
-//     fn drop(&mut self) {
-//         unsafe {
-//             cfg_if! {
-//                 if #[cfg(feature = "g4")] {
-//                     (*pac::DMA1::ptr()).ifcr.write(|w| w.gif2().clear_bit());
-//                 } else if #[cfg(feature = "g0")] {
-//                 } else if #[cfg(feature = "g0")] {
-//                     (*pac::DMA::ptr()).ifcr.write(|w| w.cgif2().clear_bit());
-//                 } else {
-//                     (*pac::DMA1::ptr()).ifcr.write(|w| w.cgif2().clear_bit());
-//                 }
-//             }
-//             cfg_if! {
-//                 if #[cfg(feature = "f3")] {
-//                     (*pac::DMA1::ptr()).ch2.cr.modify(|_, w| w.en().clear_bit());
-//                 } else if #[cfg(feature = "g0")] {
-//                     (*pac::DMA::ptr()).ch2.cr.modify(|_, w| w.en().clear_bit());
-//                 } else {
-//                     (*pac::DMA1::ptr()).ccr2.modify(|_, w| w.en().clear_bit());
-//                 }
-//             }
-//         }
-//     }
-// }
-//
-// pub struct DmaReadBuf<'a, T> {
-//     // pub buf: &'static [u8]
-//     pub buf: &'a [T],
-// }
-//
-// // unsafe impl StaticReadBuffer for DmaReadBuf {
-// //     type Word = u8;
-// //
-// //     unsafe fn static_write_buffer(&self) -> (*const Self::Word, usize) {
-// //         (self.buf[.as_ptr(), self.buf.len())
-// //     }
-// // }
-//
-// unsafe impl<'a, T> ReadBuffer for DmaReadBuf<'a, T> {
-//     type Word = T;
-//
-//     unsafe fn read_buffer(&self) -> (*const Self::Word, usize) {
-//         (self.buf.as_ptr(), self.buf.len())
-//     }
-// }
-//
-// impl<T> Drop for DmaReadBuf<'_, T> {
-//     // todo: Hardcoded for DMA1 and Chan 2.
-//     // todo: Does this stop all transfers in progress?
-//
-//     // todo: DRY with impl in DmaWriteBuf above.
-//     fn drop(&mut self) {
-//         unsafe {
-//             // Global interrupt clear flag for this channel.
-//             cfg_if! {
-//                 if #[cfg(feature = "g4")] {
-//                     (*pac::DMA1::ptr()).ifcr.write(|w| w.gif2().clear_bit());
-//                 } else if #[cfg(feature = "g0")] {
-//                     (*pac::DMA::ptr()).ifcr.write(|w| w.cgif2().clear_bit());
-//                 } else {
-//                     (*pac::DMA1::ptr()).ifcr.write(|w| w.cgif2().clear_bit());
-//                 }
-//             }
-//             cfg_if! {
-//                 if #[cfg(feature = "f3")] {
-//                     (*pac::DMA1::ptr()).ch2.cr.modify(|_, w| w.en().clear_bit());
-//                 } else if #[cfg(feature = "g0")] {
-//                     (*pac::DMA::ptr()).ch2.cr.modify(|_, w| w.en().clear_bit());
-//                 } else {
-//                     (*pac::DMA1::ptr()).ccr2.modify(|_, w| w.en().clear_bit());
-//                 }
-//             }
-//         }
-//     }
-// }
-
-#[cfg(any(feature = "l5", feature = "g0", feature = "g4", feature = "wb"))]
+#[cfg(any(
+    feature = "l5",
+    feature = "g0",
+    feature = "g4",
+    feature = "wb",
+    feature = "wl"
+))]
 /// Configure a specific DMA channel to work with a specific peripheral.
 pub fn mux(channel: DmaChannel, input: DmaInput, mux: &DMAMUX) {
     // Note: This is similar in API and purpose to `channel_select` above,
