@@ -1,24 +1,68 @@
-//! Inter-Integrated Circuit (I2C) bus. Also supports SMBUS. Implements traits from `embedded-hal`.
+//! Inter-Integrated Circuit (I2C) bus. Also supports SMBUS. Provides APIs to configure, read, and write from
+//! I2C, with blocking, nonblocking, and DMA functionality.
 
 use cast::{u16, u8};
 use core::ops::Deref;
 
+use cortex_m::interrupt::free;
+
+#[cfg(feature = "embedded-hal")]
 use embedded_hal::blocking::i2c::{Read, Write, WriteRead};
 
 use crate::{
+    clocks::Clocks,
     pac::{self, RCC},
     rcc_en_reset,
-    traits::ClockCfg,
 };
 
 #[cfg(feature = "g0")]
 use crate::pac::dma as dma_p;
-#[cfg(any(feature = "f3", feature = "l4", feature = "g4", feature = "wb"))]
+#[cfg(any(
+    feature = "f3",
+    feature = "l4",
+    feature = "g4",
+    feature = "wb",
+    feature = "wl"
+))]
 use crate::pac::dma1 as dma_p;
 
 #[cfg(not(any(feature = "h7", feature = "f4", feature = "l5")))]
 use crate::dma::{self, Dma, DmaChannel, DmaInput};
 
+macro_rules! busy_wait {
+    ($regs:expr, $flag:ident, $variant:ident) => {
+        loop {
+            let isr = $regs.isr.read();
+
+            if isr.$flag().$variant() {
+                break;
+            } else if isr.berr().bit_is_set() {
+                $regs.icr.write(|w| w.berrcf().set_bit());
+                return Err(Error::Bus);
+            } else if isr.arlo().bit_is_set() {
+                $regs.icr.write(|w| w.arlocf().set_bit());
+                return Err(Error::Arbitration);
+            } else if isr.nackf().bit_is_set() {
+                $regs.icr.write(|w| w.stopcf().set_bit().nackcf().set_bit());
+
+                // If a pending TXIS flag is set, write dummy data to TXDR
+                if $regs.isr.read().txis().bit_is_set() {
+                    $regs.txdr.write(|w| unsafe { w.txdata().bits(0) });
+                }
+
+                // If TXDR is not flagged as empty, write 1 to flush it
+                if $regs.isr.read().txe().bit_is_clear() {
+                    $regs.isr.write(|w| w.txe().set_bit());
+                }
+
+                return Err(Error::Nack);
+            } else {
+                // try again
+            }
+        }
+    };
+}
+
 /// I2C error
 #[non_exhaustive]
 #[derive(Debug)]
@@ -46,46 +90,49 @@ pub enum Error {
 #[derive(Clone, Copy)]
 pub enum I2cDevice {
     One,
-    #[cfg(not(feature = "wb"))]
+    #[cfg(not(any(feature = "wb", feature = "f3x4")))]
     Two,
     #[cfg(any(feature = "h7", feature = "wb"))]
     Three,
 }
 
 /// Represents an Inter-Integrated Circuit (I2C) peripheral.
-pub struct I2c<I> {
-    regs: I,
+pub struct I2c<R> {
+    regs: R,
     device: I2cDevice,
     // mode: I2cMode,
     /// SMBUS features like PEC enabled.
     smbus: bool,
 }
 
-impl<I> I2c<I>
+impl<R> I2c<R>
 where
-    I: Deref<Target = pac::i2c1::RegisterBlock>,
+    R: Deref<Target = pac::i2c1::RegisterBlock>,
 {
     /// Configures the I2C peripheral. `freq` is in Hz. Doesn't check pin config.
-    pub fn new<C: ClockCfg>(
-        regs: I,
+    pub fn new(
+        regs: R,
         device: I2cDevice,
-        freq: u32,
-        clocks: &C,
-        rcc: &mut RCC,
+        freq: u32, // todo: Set a division manually, like you do with SPI.
+        clocks: &Clocks,
     ) -> Self {
-        match device {
-            I2cDevice::One => {
-                rcc_en_reset!(apb1, i2c1, rcc);
+        free(|cs| {
+            let mut rcc = unsafe { &(*RCC::ptr()) };
+
+            match device {
+                I2cDevice::One => {
+                    rcc_en_reset!(apb1, i2c1, rcc);
+                }
+                #[cfg(not(any(feature = "wb", feature = "f3x4")))]
+                I2cDevice::Two => {
+                    rcc_en_reset!(apb1, i2c2, rcc);
+                }
+                #[cfg(any(feature = "h7", feature = "wb"))]
+                I2cDevice::Three => {
+                    rcc_en_reset!(apb1, i2c3, rcc);
+                }
             }
-            #[cfg(not(feature = "wb"))]
-            I2cDevice::Two => {
-                rcc_en_reset!(apb1, i2c2, rcc);
-            }
-            #[cfg(any(feature = "h7", feature = "wb"))]
-            I2cDevice::Three => {
-                rcc_en_reset!(apb1, i2c3, rcc);
-            }
-        }
+        });
 
         assert!(freq <= 1_000_000);
         // Make sure the I2C unit is disabled so we can configure it
@@ -196,6 +243,86 @@ where
         }
     }
 
+    /// Read multiple words.
+    pub fn read(&mut self, addr: u8, bytes: &mut [u8]) -> Result<(), Error> {
+        // Wait for any previous address sequence to end
+        // automatically. This could be up to 50% of a bus
+        // cycle (ie. up to 0.5/freq)
+        while self.regs.cr2.read().start().bit_is_set() {}
+
+        // Set START and prepare to receive bytes into
+        // `buffer`. The START bit can be set even if the bus
+        // is BUSY or I2C is in slave mode.
+        self.set_cr2_read(addr, bytes.len() as u8);
+
+        for byte in bytes {
+            // Wait until we have received something
+            busy_wait!(self.regs, rxne, bit_is_set);
+
+            *byte = self.regs.rxdr.read().rxdata().bits();
+        }
+
+        Ok(())
+    }
+
+    /// Write an array of words
+    pub fn write(&mut self, addr: u8, bytes: &[u8]) -> Result<(), Error> {
+        // Wait for any previous address sequence to end
+        // automatically. This could be up to 50% of a bus
+        // cycle (ie. up to 0.5/freq)
+        while self.regs.cr2.read().start().bit_is_set() {}
+
+        self.set_cr2_write(addr, bytes.len() as u8, true);
+
+        for byte in bytes {
+            // Wait until we are allowed to send data
+            // (START has been ACKed or last byte when
+            // through)
+            busy_wait!(self.regs, txis, bit_is_set); // TXDR register is empty
+
+            // Put byte on the wire
+            self.regs.txdr.write(|w| unsafe { w.txdata().bits(*byte) });
+        }
+
+        Ok(())
+    }
+
+    /// Write and read an array of words.
+    pub fn write_read(&mut self, addr: u8, bytes: &[u8], buffer: &mut [u8]) -> Result<(), Error> {
+        // Wait for any previous address sequence to end
+        // automatically. This could be up to 50% of a bus
+        // cycle (ie. up to 0.5/freq)
+        while self.regs.cr2.read().start().bit_is_set() {}
+
+        self.set_cr2_write(addr, bytes.len() as u8, false);
+
+        for byte in bytes {
+            // Wait until we are allowed to send data
+            // (START has been ACKed or last byte went through)
+
+            busy_wait!(self.regs, txis, bit_is_set); // TXDR register is empty
+
+            // Put byte on the wire
+            self.regs.txdr.write(|w| unsafe { w.txdata().bits(*byte) });
+        }
+
+        // Wait until the write finishes before beginning to read.
+        busy_wait!(self.regs, tc, bit_is_set); // transfer is complete
+
+        // reSTART and prepare to receive bytes into `buffer`
+
+        self.set_cr2_read(addr, buffer.len() as u8);
+
+        for byte in buffer {
+            // Wait until we have received something
+            busy_wait!(self.regs, rxne, bit_is_set);
+
+            *byte = self.regs.rxdr.read().rxdata().bits();
+        }
+
+        Ok(())
+    }
+
     /// Helper function to prevent repetition between `write`, `write_read`, and `write_dma`.
     fn set_cr2_write(&mut self, addr: u8, len: u8, autoend: bool) {
         // L44 RM: "Master communication initialization (address phase)
@@ -261,7 +388,7 @@ where
     /// Note that the `channel` argument is only used on F3 and L4.
     /// For a single write, set `autoend` to `true`. For a write_read and other use cases,
     /// set it to `false`.
-    pub fn write_dma<D>(
+    pub unsafe fn write_dma<D>(
         &mut self,
         addr: u8,
         buf: &[u8],
@@ -279,7 +406,7 @@ where
         #[cfg(any(feature = "f3", feature = "l4"))]
         let channel = match self.device {
             I2cDevice::One => DmaInput::I2c1Tx.dma1_channel(),
-            #[cfg(not(feature = "wb"))]
+            #[cfg(not(any(feature = "wb", feature = "f3x4")))]
             I2cDevice::Two => DmaInput::I2c2Tx.dma1_channel(),
             #[cfg(any(feature = "h7", feature = "wb"))]
             I2cDevice::Three => DmaInput::I2c3Tx.dma1_channel(),
@@ -346,8 +473,13 @@ where
     #[cfg(not(any(feature = "g0", feature = "h7", feature = "f4", feature = "l5")))]
     /// Read data, using DMA. See L44 RM, 37.4.16: "Reception using DMA"
     /// Note that the `channel` argument is only used on F3 and L4.
-    pub fn read_dma<D>(&mut self, addr: u8, buf: &mut [u8], channel: DmaChannel, dma: &mut Dma<D>)
-    where
+    pub unsafe fn read_dma<D>(
+        &mut self,
+        addr: u8,
+        buf: &mut [u8],
+        channel: DmaChannel,
+        dma: &mut Dma<D>,
+    ) where
         D: Deref<Target = dma_p::RegisterBlock>,
     {
         // while self.regs.cr2.read().start().bit_is_set() {}
@@ -359,7 +491,7 @@ where
         #[cfg(any(feature = "f3", feature = "l4"))]
         let channel = match self.device {
             I2cDevice::One => DmaInput::I2c1Rx.dma1_channel(),
-            #[cfg(not(feature = "wb"))]
+            #[cfg(not(any(feature = "wb", feature = "f3x4")))]
             I2cDevice::Two => DmaInput::I2c2Rx.dma1_channel(),
             #[cfg(any(feature = "h7", feature = "wb"))]
             I2cDevice::Three => DmaInput::I231Rx.dma1_channel(),
@@ -406,134 +538,41 @@ where
     }
 }
 
-macro_rules! busy_wait {
-    ($regs:expr, $flag:ident, $variant:ident) => {
-        loop {
-            let isr = $regs.isr.read();
-
-            if isr.$flag().$variant() {
-                break;
-            } else if isr.berr().bit_is_set() {
-                $regs.icr.write(|w| w.berrcf().set_bit());
-                return Err(Error::Bus);
-            } else if isr.arlo().bit_is_set() {
-                $regs.icr.write(|w| w.arlocf().set_bit());
-                return Err(Error::Arbitration);
-            } else if isr.nackf().bit_is_set() {
-                $regs.icr.write(|w| w.stopcf().set_bit().nackcf().set_bit());
-
-                // If a pending TXIS flag is set, write dummy data to TXDR
-                if $regs.isr.read().txis().bit_is_set() {
-                    $regs.txdr.write(|w| unsafe { w.txdata().bits(0) });
-                }
-
-                // If TXDR is not flagged as empty, write 1 to flush it
-                if $regs.isr.read().txe().bit_is_clear() {
-                    $regs.isr.write(|w| w.txe().set_bit());
-                }
-
-                return Err(Error::Nack);
-            } else {
-                // try again
-            }
-        }
-    };
-}
-
-impl<I2C> Write for I2c<I2C>
+#[cfg(feature = "embedded-hal")]
+#[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
+impl<R> Write for I2c<R>
 where
-    I2C: Deref<Target = pac::i2c1::RegisterBlock>,
+    R: Deref<Target = pac::i2c1::RegisterBlock>,
 {
     type Error = Error;
 
     fn write(&mut self, addr: u8, bytes: &[u8]) -> Result<(), Error> {
-        // Wait for any previous address sequence to end
-        // automatically. This could be up to 50% of a bus
-        // cycle (ie. up to 0.5/freq)
-        while self.regs.cr2.read().start().bit_is_set() {}
-
-        self.set_cr2_write(addr, bytes.len() as u8, true);
-
-        for byte in bytes {
-            // Wait until we are allowed to send data
-            // (START has been ACKed or last byte when
-            // through)
-            busy_wait!(self.regs, txis, bit_is_set); // TXDR register is empty
-
-            // Put byte on the wire
-            self.regs.txdr.write(|w| unsafe { w.txdata().bits(*byte) });
-        }
-
-        Ok(())
+        I2c::write(self, addr, bytes)
     }
 }
 
-impl<I2C> Read for I2c<I2C>
+#[cfg(feature = "embedded-hal")]
+#[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
+impl<R> Read for I2c<R>
 where
-    I2C: Deref<Target = pac::i2c1::RegisterBlock>,
+    R: Deref<Target = pac::i2c1::RegisterBlock>,
 {
     type Error = Error;
 
     fn read(&mut self, addr: u8, bytes: &mut [u8]) -> Result<(), Error> {
-        // Wait for any previous address sequence to end
-        // automatically. This could be up to 50% of a bus
-        // cycle (ie. up to 0.5/freq)
-        while self.regs.cr2.read().start().bit_is_set() {}
-
-        // Set START and prepare to receive bytes into
-        // `buffer`. The START bit can be set even if the bus
-        // is BUSY or I2C is in slave mode.
-        self.set_cr2_read(addr, bytes.len() as u8);
-
-        for byte in bytes {
-            // Wait until we have received something
-            busy_wait!(self.regs, rxne, bit_is_set);
-
-            *byte = self.regs.rxdr.read().rxdata().bits();
-        }
-
-        Ok(())
+        I2c::read(self, addr, bytes)
     }
 }
 
-impl<I2C> WriteRead for I2c<I2C>
+#[cfg(feature = "embedded-hal")]
+#[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
+impl<R> WriteRead for I2c<R>
 where
-    I2C: Deref<Target = pac::i2c1::RegisterBlock>,
+    R: Deref<Target = pac::i2c1::RegisterBlock>,
 {
     type Error = Error;
 
     fn write_read(&mut self, addr: u8, bytes: &[u8], buffer: &mut [u8]) -> Result<(), Error> {
-        // Wait for any previous address sequence to end
-        // automatically. This could be up to 50% of a bus
-        // cycle (ie. up to 0.5/freq)
-        while self.regs.cr2.read().start().bit_is_set() {}
-
-        self.set_cr2_write(addr, bytes.len() as u8, false);
-
-        for byte in bytes {
-            // Wait until we are allowed to send data
-            // (START has been ACKed or last byte went through)
-
-            busy_wait!(self.regs, txis, bit_is_set); // TXDR register is empty
-
-            // Put byte on the wire
-            self.regs.txdr.write(|w| unsafe { w.txdata().bits(*byte) });
-        }
-
-        // Wait until the write finishes before beginning to read.
-        busy_wait!(self.regs, tc, bit_is_set); // transfer is complete
-
-        // reSTART and prepare to receive bytes into `buffer`
-
-        self.set_cr2_read(addr, buffer.len() as u8);
-
-        for byte in buffer {
-            // Wait until we have received something
-            busy_wait!(self.regs, rxne, bit_is_set);
-
-            *byte = self.regs.rxdr.read().rxdata().bits();
-        }
-
-        Ok(())
+        I2c::write_read(self, addr, bytes, buffer)
     }
 }
