@@ -2,11 +2,14 @@
 //! For more details, see
 //! [ST AN4759](https:/www.st.com%2Fresource%2Fen%2Fapplication_note%2Fdm00226326-using-the-hardware-realtime-clock-rtc-and-the-tamper-management-unit-tamp-with-stm32-microcontrollers-stmicroelectronics.pdf&usg=AOvVaw3PzvL2TfYtwS32fw-Uv37h)
 
-// Based on `stm32f3xx-hal`.
+//! Uses [Chrono](https://docs.rs/chrono) for dates and times.
 
 use crate::pac::{EXTI, PWR, RCC, RTC};
 use core::convert::TryInto;
-use rtcc::{Datelike, Hours, NaiveDate, NaiveDateTime, NaiveTime, Rtcc, Timelike};
+
+use cortex_m::interrupt::free;
+
+use chrono::{Datelike, NaiveDate, NaiveDateTime, NaiveTime, Timelike};
 
 use cfg_if::cfg_if;
 
@@ -115,7 +118,7 @@ impl Rtc {
     /// doesn't connect to `OSC32_IN`, such as a MEMS resonator.
     /// Note that if using HSE as the clock source, we assume you've already enabled it, eg
     /// in clock config.
-    pub fn new(regs: RTC, rcc: &mut RCC, pwr: &mut PWR, config: RtcConfig) -> Self {
+    pub fn new(regs: RTC, pwr: &mut PWR, config: RtcConfig) -> Self {
         let mut result = Self { regs, config };
 
         // Enable the peripheral clock for communication
@@ -126,82 +129,85 @@ impl Rtc {
         // field here.
 
         // See L4 RM, `Backup domain access` section.
+        free(|cs| {
+            let mut rcc = unsafe { &(*RCC::ptr()) };
 
-        cfg_if! {
-            if #[cfg(any(feature = "f3", feature = "f4"))] {
-                rcc.apb1enr.modify(|_, w| w.pwren().set_bit());
-                pwr.cr.read(); // read to allow the pwr clock to enable
-                pwr.cr.modify(|_, w| w.dbp().set_bit());
-                while pwr.cr.read().dbp().bit_is_clear() {}
-            } else if #[cfg(any(feature = "l4", feature = "l5", feature = "g4", feature = "wb"))] {
-                // 1. Enable the power interface clock by setting the PWREN bits in the Section 6.4.18:
-                // APB1 peripheral clock enable register 1 (RCC_APB1ENR1)
-                #[cfg(not(feature = "wb"))]
-                rcc.apb1enr1.modify(|_, w| {
-                    w.pwren().set_bit();
-                    w.rtcapben().set_bit()
-                });
-                #[cfg(feature = "wb")]
-                rcc.apb1enr1.modify(|_, w| w.rtcapben().set_bit());
+            cfg_if! {
+                if #[cfg(any(feature = "f3", feature = "f4"))] {
+                    rcc.apb1enr.modify(|_, w| w.pwren().set_bit());
+                    pwr.cr.read(); // read to allow the pwr clock to enable
+                    pwr.cr.modify(|_, w| w.dbp().set_bit());
+                    while pwr.cr.read().dbp().bit_is_clear() {}
+                } else if #[cfg(any(feature = "l4", feature = "l5", feature = "g4", feature = "l412", feature = "wb", feature = "wl"))] {
+                    // 1. Enable the power interface clock by setting the PWREN bits in the Section 6.4.18:
+                    // APB1 peripheral clock enable register 1 (RCC_APB1ENR1)
+                    #[cfg(not(any(feature = "wb", feature = "wl")))]
+                    rcc.apb1enr1.modify(|_, w| {
+                        w.pwren().set_bit();
+                        w.rtcapben().set_bit()
+                    });
+                    #[cfg(any(feature = "wb", feature = "wl"))]
+                    rcc.apb1enr1.modify(|_, w| w.rtcapben().set_bit());
 
-                rcc.apb1smenr1.modify(|_, w| w.rtcapbsmen().set_bit());  // In sleep and stop modes.
-                pwr.cr1.read(); // Read to allow the pwr clock to enable
-                // 2. Set the DBP bit in the Power control register 1 (PWR_CR1) to enable access to the
-                // backup domain
-                pwr.cr1.modify( | _, w| w.dbp().set_bit()); // Unlock the backup domain
-                while pwr.cr1.read().dbp().bit_is_clear() {}
-            } else if #[cfg(any(feature = "g0"))] {
-                rcc.apbenr1.modify(|_, w| {
-                    w.pwren().set_bit();
-                    w.rtcapben().set_bit()
-                });
-                rcc.apbsmenr1.modify(|_, w| w.rtcapbsmen().set_bit());  // In sleep and stop modes.
-                pwr.cr1.read();
-                pwr.cr1.modify( | _, w| w.dbp().set_bit());
-                while pwr.cr1.read().dbp().bit_is_clear() {}
-            } else { // eg h7
-                rcc.apb4enr.modify(|_, w| w.rtcapben().set_bit());
-                rcc.apb4lpenr.modify(|_, w| w.rtcapblpen().set_bit());  // In sleep and stop modes.
-                pwr.cr1.read(); // read to allow the pwr clock to enable
-                pwr.cr1.modify( | _, w| w.dbp().set_bit());
-                while pwr.cr1.read().dbp().bit_is_clear() {}
-            }
-        }
-
-        // Reset the backup domain.
-        rcc.bdcr.modify(|_, w| w.bdrst().set_bit());
-        rcc.bdcr.modify(|_, w| w.bdrst().clear_bit());
-
-        // Set up the LSI or LSE as required.
-        match config.clock_source {
-            RtcClockSource::Lsi => {
-                cfg_if! {
-                    if #[cfg(feature = "wb")] {
-                    // todo: LSI2?
-                        rcc.csr.modify(|_, w| w.lsi1on().set_bit());
-                        while rcc.csr.read().lsi1rdy().bit_is_clear() {}
-                    } else {
-                        rcc.csr.modify(|_, w| w.lsion().set_bit());
-                        while rcc.csr.read().lsirdy().bit_is_clear() {}
-                    }
+                    rcc.apb1smenr1.modify(|_, w| w.rtcapbsmen().set_bit());  // In sleep and stop modes.
+                    pwr.cr1.read(); // Read to allow the pwr clock to enable
+                    // 2. Set the DBP bit in the Power control register 1 (PWR_CR1) to enable access to the
+                    // backup domain
+                    pwr.cr1.modify( | _, w| w.dbp().set_bit()); // Unlock the backup domain
+                    while pwr.cr1.read().dbp().bit_is_clear() {}
+                } else if #[cfg(any(feature = "g0"))] {
+                    rcc.apbenr1.modify(|_, w| {
+                        w.pwren().set_bit();
+                        w.rtcapben().set_bit()
+                    });
+                    rcc.apbsmenr1.modify(|_, w| w.rtcapbsmen().set_bit());  // In sleep and stop modes.
+                    pwr.cr1.read();
+                    pwr.cr1.modify( | _, w| w.dbp().set_bit());
+                    while pwr.cr1.read().dbp().bit_is_clear() {}
+                } else { // eg h7
+                    rcc.apb4enr.modify(|_, w| w.rtcapben().set_bit());
+                    rcc.apb4lpenr.modify(|_, w| w.rtcapblpen().set_bit());  // In sleep and stop modes.
+                    pwr.cr1.read(); // read to allow the pwr clock to enable
+                    pwr.cr1.modify( | _, w| w.dbp().set_bit());
+                    while pwr.cr1.read().dbp().bit_is_clear() {}
                 }
             }
-            RtcClockSource::Lse => {
-                // Can only set lsebyp when lse is off, so do this as a separate step.
-                rcc.bdcr
-                    .modify(|_, w| w.lsebyp().bit(config.bypass_lse_output));
-                rcc.bdcr.modify(|_, w| w.lseon().set_bit());
-                while rcc.bdcr.read().lserdy().bit_is_clear() {}
-            }
-            _ => (),
-        }
 
-        // 3. Select the RTC clock source in the Backup domain control register (RCC_BDCR).
-        // 4. Enable the RTC clock by setting the RTCEN [15] bit in the Backup domain control
-        // register (RCC_BDCR)
-        rcc.bdcr.modify(|_, w| {
-            unsafe { w.rtcsel().bits(result.config.clock_source as u8) };
-            w.rtcen().set_bit()
+            // Reset the backup domain.
+            rcc.bdcr.modify(|_, w| w.bdrst().set_bit());
+            rcc.bdcr.modify(|_, w| w.bdrst().clear_bit());
+
+            // Set up the LSI or LSE as required.
+            match config.clock_source {
+                RtcClockSource::Lsi => {
+                    cfg_if! {
+                        if #[cfg(any(feature = "wb", feature = "wl"))] {
+                        // todo: LSI2?
+                            rcc.csr.modify(|_, w| w.lsi1on().set_bit());
+                            while rcc.csr.read().lsi1rdy().bit_is_clear() {}
+                        } else {
+                            rcc.csr.modify(|_, w| w.lsion().set_bit());
+                            while rcc.csr.read().lsirdy().bit_is_clear() {}
+                        }
+                    }
+                }
+                RtcClockSource::Lse => {
+                    // Can only set lsebyp when lse is off, so do this as a separate step.
+                    rcc.bdcr
+                        .modify(|_, w| w.lsebyp().bit(config.bypass_lse_output));
+                    rcc.bdcr.modify(|_, w| w.lseon().set_bit());
+                    while rcc.bdcr.read().lserdy().bit_is_clear() {}
+                }
+                _ => (),
+            }
+
+            // 3. Select the RTC clock source in the Backup domain control register (RCC_BDCR).
+            // 4. Enable the RTC clock by setting the RTCEN [15] bit in the Backup domain control
+            // register (RCC_BDCR)
+            rcc.bdcr.modify(|_, w| {
+                unsafe { w.rtcsel().bits(result.config.clock_source as u8) };
+                w.rtcen().set_bit()
+            });
         });
 
         result.edit_regs(false, |regs| {
@@ -253,8 +259,8 @@ impl Rtc {
     // pub fn set_alarm(&mut self, exti: &mut EXTI) {
     // note: STM3241x and 42x have diff addresses, and are PAC incompatible!
     //     exti.imr1.modify(|_, w| w.mr18().unmasked());
-    //     exti.rtsr1.modify(|_, w| w.tr18().bit(true));
-    //     exti.ftsr1.modify(|_, w| w.tr18().bit(false));
+    //     exti.rtsr1.modify(|_, w| w.tr18().set_bit());
+    //     exti.ftsr1.modify(|_, w| w.tr18().clear_bit());
     //
     //     self.edit_regs(false, |regs| {
     //         regs.cr.modify(|_, w| w.alrae().clear_bit());
@@ -389,29 +395,29 @@ impl Rtc {
         cfg_if! {
             if #[cfg(any(feature = "f3", feature = "l4"))] {
                 exti.imr1.modify(|_, w| w.mr20().unmasked());
-                exti.rtsr1.modify(|_, w| w.tr20().bit(true));
-                exti.ftsr1.modify(|_, w| w.tr20().bit(false));
+                exti.rtsr1.modify(|_, w| w.tr20().set_bit());
+                exti.ftsr1.modify(|_, w| w.tr20().clear_bit());
             } else if #[cfg(feature = "f4")] {
                 exti.imr.modify(|_, w| w.mr20().unmasked());
-                exti.rtsr.modify(|_, w| w.tr20().bit(true));
-                exti.ftsr.modify(|_, w| w.tr20().bit(false));
+                exti.rtsr.modify(|_, w| w.tr20().set_bit());
+                exti.ftsr.modify(|_, w| w.tr20().clear_bit());
             } else if #[cfg(feature = "g4")]{
                 exti.imr1.modify(|_, w| w.im20().unmasked());
-                exti.rtsr1.modify(|_, w| w.rt20().bit(true));
-                exti.ftsr1.modify(|_, w| w.ft20().bit(false));
-            } else if #[cfg(any(feature = "l5", feature = "g0", feature = "wb"))] {
+                exti.rtsr1.modify(|_, w| w.rt20().set_bit());
+                exti.ftsr1.modify(|_, w| w.ft20().clear_bit());
+            } else if #[cfg(any(feature = "l5", feature = "g0", feature = "wb", feature = "wl"))] {
                 // exti.imr1.modify(|_, w| w.mr20().unmasked());
-                // exti.rtsr1.modify(|_, w| w.rt20().bit(true));
-                // exti.ftsr1.modify(|_, w| w.ft20().bit(false));
+                // exti.rtsr1.modify(|_, w| w.rt20().set_bit());
+                // exti.ftsr1.modify(|_, w| w.ft20().clear_bit());
 
            } else if #[cfg(any(feature = "h747cm4", feature = "h747cm7"))] {
                 exti.c1imr1.modify(|_, w| w.mr20().unmasked());
-                exti.rtsr1.modify(|_, w| w.tr20().bit(true));
-                exti.ftsr1.modify(|_, w| w.tr20().bit(false));
+                exti.rtsr1.modify(|_, w| w.tr20().set_bit());
+                exti.ftsr1.modify(|_, w| w.tr20().clear_bit());
            } else { // H7
                 exti.cpuimr1.modify(|_, w| w.mr20().unmasked());
-                exti.rtsr1.modify(|_, w| w.tr20().bit(true));
-                exti.ftsr1.modify(|_, w| w.tr20().bit(false));
+                exti.rtsr1.modify(|_, w| w.tr20().set_bit());
+                exti.ftsr1.modify(|_, w| w.tr20().clear_bit());
             }
         }
 
@@ -426,7 +432,7 @@ impl Rtc {
         // Ensure access to Wakeup auto-reload counter and bits WUCKSEL[2:0] is allowed.
         // Poll WUTWF until it is set in RTC_ISR (RTC2)/RTC_ICSR (RTC3) (May not be avail on F3)
         cfg_if! {
-            if #[cfg(any(feature = "l5", feature = "g0", feature = "g4"))] {
+            if #[cfg(any(feature = "l5", feature = "g0", feature = "g4", feature = "l412"))] {
                 while self.regs.icsr.read().wutwf().bit_is_clear() {}
             } else {
                 while self.regs.isr.read().wutwf().bit_is_clear() {}
@@ -442,7 +448,7 @@ impl Rtc {
         self.regs.cr.modify(|_, w| w.wutie().set_bit());
 
         cfg_if! {
-            if #[cfg(any(feature = "l5", feature = "g0", feature = "g4"))] {
+            if #[cfg(any(feature = "l412", feature = "l5", feature = "g0", feature = "g4", feature = "l412"))] {
                 self.regs.scr.write(|w| w.cwutf().set_bit());
             } else {
                 self.regs.isr.modify(|_, w| w.wutf().clear_bit());
@@ -490,7 +496,7 @@ impl Rtc {
         }
 
         cfg_if! {
-            if #[cfg(any(feature = "l5", feature = "g0", feature = "g4"))] {
+            if #[cfg(any(feature = "l5", feature = "g0", feature = "g4", feature = "l412"))] {
                 while self.regs.icsr.read().wutwf().bit_is_clear() {}
             } else {
                 while self.regs.isr.read().wutwf().bit_is_clear() {}
@@ -513,9 +519,12 @@ impl Rtc {
             regs.cr.modify(|_, w| w.wute().clear_bit());
 
             cfg_if! {
-                if #[cfg(any(feature = "l5", feature = "g0", feature = "g4"))] {
+                if #[cfg(any(feature = "l412", feature = "l5", feature = "g0", feature = "g4", feature = "l412"))] {
                     regs.scr.write(|w| w.cwutf().set_bit());
                 } else {
+                    // Note that we clear this by writing 0, which isn't
+                    // the standard convention, eg in other families, and
+                    // other peripherals.
                     regs.isr.modify(|_, w| w.wutf().clear_bit());
                 }
             }
@@ -539,7 +548,7 @@ impl Rtc {
         // todo: L4 has ICSR and ISR regs. Maybe both for backwards compat?
 
         cfg_if! {
-             if #[cfg(any(feature = "l5", feature = "g0", feature = "g4"))] {
+             if #[cfg(any(feature = "l5", feature = "g0", feature = "g4", feature = "l412"))] {
                  // Enter init mode if required. This is generally used to edit the clock or calendar,
                  // but not for initial enabling steps.
                  if init_mode && self.regs.icsr.read().initf().bit_is_clear() {
@@ -574,14 +583,10 @@ impl Rtc {
         // This is safe, as the field accepts the full range of 8-bit values.
         self.regs.wpr.write(|w| unsafe { w.bits(0xFF) });
     }
-}
-
-impl Rtcc for Rtc {
-    type Error = Error;
 
     /// set time using NaiveTime (ISO 8601 time without timezone)
     /// Hour format is 24h
-    fn set_time(&mut self, time: &NaiveTime) -> Result<(), Self::Error> {
+    pub fn set_time(&mut self, time: &NaiveTime) -> Result<(), Error> {
         self.set_24h_fmt();
         let (ht, hu) = bcd2_encode(time.hour())?;
         let (mnt, mnu) = bcd2_encode(time.minute())?;
@@ -602,7 +607,7 @@ impl Rtcc for Rtc {
         Ok(())
     }
 
-    fn set_seconds(&mut self, seconds: u8) -> Result<(), Self::Error> {
+    pub fn set_seconds(&mut self, seconds: u8) -> Result<(), Error> {
         if seconds > 59 {
             return Err(Error::InvalidInputData);
         }
@@ -615,7 +620,7 @@ impl Rtcc for Rtc {
         Ok(())
     }
 
-    fn set_minutes(&mut self, minutes: u8) -> Result<(), Self::Error> {
+    pub fn set_minutes(&mut self, minutes: u8) -> Result<(), Error> {
         if minutes > 59 {
             return Err(Error::InvalidInputData);
         }
@@ -628,12 +633,8 @@ impl Rtcc for Rtc {
         Ok(())
     }
 
-    fn set_hours(&mut self, hours: Hours) -> Result<(), Self::Error> {
-        let (ht, hu) = hours_to_register(hours)?;
-        match hours {
-            Hours::H24(_h) => self.set_24h_fmt(),
-            Hours::AM(_h) | Hours::PM(_h) => self.set_12h_fmt(),
-        }
+    pub fn set_hours(&mut self, hours: u8) -> Result<(), Error> {
+        let (ht, hu) = bcd2_encode(hours as u32)?;
 
         self.edit_regs(true, |regs| {
             regs.tr
@@ -643,7 +644,7 @@ impl Rtcc for Rtc {
         Ok(())
     }
 
-    fn set_weekday(&mut self, weekday: u8) -> Result<(), Self::Error> {
+    pub fn set_weekday(&mut self, weekday: u8) -> Result<(), Error> {
         if !(1..=7).contains(&weekday) {
             return Err(Error::InvalidInputData);
         }
@@ -654,7 +655,7 @@ impl Rtcc for Rtc {
         Ok(())
     }
 
-    fn set_day(&mut self, day: u8) -> Result<(), Self::Error> {
+    pub fn set_day(&mut self, day: u8) -> Result<(), Error> {
         if !(1..=31).contains(&day) {
             return Err(Error::InvalidInputData);
         }
@@ -667,7 +668,7 @@ impl Rtcc for Rtc {
         Ok(())
     }
 
-    fn set_month(&mut self, month: u8) -> Result<(), Self::Error> {
+    pub fn set_month(&mut self, month: u8) -> Result<(), Error> {
         if !(1..=12).contains(&month) {
             return Err(Error::InvalidInputData);
         }
@@ -680,7 +681,7 @@ impl Rtcc for Rtc {
         Ok(())
     }
 
-    fn set_year(&mut self, year: u16) -> Result<(), Self::Error> {
+    pub fn set_year(&mut self, year: u16) -> Result<(), Error> {
         if !(1970..=2038).contains(&year) {
             return Err(Error::InvalidInputData);
         }
@@ -695,7 +696,7 @@ impl Rtcc for Rtc {
 
     /// Set the date using NaiveDate (ISO 8601 calendar date without timezone).
     /// WeekDay is set using the `set_weekday` method
-    fn set_date(&mut self, date: &NaiveDate) -> Result<(), Self::Error> {
+    pub fn set_date(&mut self, date: &NaiveDate) -> Result<(), Error> {
         if date.year() < 1970 {
             return Err(Error::InvalidInputData);
         }
@@ -718,7 +719,7 @@ impl Rtcc for Rtc {
         Ok(())
     }
 
-    fn set_datetime(&mut self, date: &NaiveDateTime) -> Result<(), Self::Error> {
+    pub fn set_datetime(&mut self, date: &NaiveDateTime) -> Result<(), Error> {
         if date.year() < 1970 {
             return Err(Error::InvalidInputData);
         }
@@ -758,93 +759,68 @@ impl Rtcc for Rtc {
         Ok(())
     }
 
-    fn get_seconds(&mut self) -> Result<u8, Self::Error> {
+    pub fn get_seconds(&mut self) -> u8 {
         let tr = self.regs.tr.read();
-        let seconds = bcd2_decode(tr.st().bits(), tr.su().bits());
-        Ok(seconds as u8)
+        bcd2_decode(tr.st().bits(), tr.su().bits()) as u8
     }
 
-    fn get_minutes(&mut self) -> Result<u8, Self::Error> {
+    pub fn get_minutes(&mut self) -> u8 {
         let tr = self.regs.tr.read();
-        let minutes = bcd2_decode(tr.mnt().bits(), tr.mnu().bits());
-        Ok(minutes as u8)
+        bcd2_decode(tr.mnt().bits(), tr.mnu().bits()) as u8
     }
 
-    fn get_hours(&mut self) -> Result<Hours, Self::Error> {
+    pub fn get_hours(&mut self) -> u8 {
         let tr = self.regs.tr.read();
-        let hours = bcd2_decode(tr.ht().bits(), tr.hu().bits());
-        if self.is_24h_fmt() {
-            return Ok(Hours::H24(hours as u8));
-        }
-        if !tr.pm().bit() {
-            return Ok(Hours::AM(hours as u8));
-        }
-        Ok(Hours::PM(hours as u8))
+        bcd2_decode(tr.ht().bits(), tr.hu().bits()) as u8
     }
 
-    fn get_time(&mut self) -> Result<NaiveTime, Self::Error> {
-        self.set_24h_fmt();
-        let seconds = self.get_seconds()?;
-        let minutes = self.get_minutes()?;
-        let hours = hours_to_u8(self.get_hours()?)?;
-
-        Ok(NaiveTime::from_hms(
-            hours.into(),
-            minutes.into(),
-            seconds.into(),
-        ))
+    pub fn get_time(&mut self) -> NaiveTime {
+        NaiveTime::from_hms(
+            self.get_hours().into(),
+            self.get_minutes().into(),
+            self.get_seconds().into(),
+        )
     }
 
-    fn get_weekday(&mut self) -> Result<u8, Self::Error> {
+    pub fn get_weekday(&mut self) -> u8 {
         let dr = self.regs.dr.read();
-        let weekday = bcd2_decode(dr.wdu().bits(), 0x00);
-        Ok(weekday as u8)
+        bcd2_decode(dr.wdu().bits(), 0x00) as u8
     }
 
-    fn get_day(&mut self) -> Result<u8, Self::Error> {
+    pub fn get_day(&mut self) -> u8 {
         let dr = self.regs.dr.read();
-        let day = bcd2_decode(dr.dt().bits(), dr.du().bits());
-        Ok(day as u8)
+        bcd2_decode(dr.dt().bits(), dr.du().bits()) as u8
     }
 
-    fn get_month(&mut self) -> Result<u8, Self::Error> {
+    pub fn get_month(&mut self) -> u8 {
         let dr = self.regs.dr.read();
         let mt: u8 = if dr.mt().bit() { 1 } else { 0 };
-        let month = bcd2_decode(mt, dr.mu().bits());
-        Ok(month as u8)
+        bcd2_decode(mt, dr.mu().bits()) as u8
     }
 
-    fn get_year(&mut self) -> Result<u16, Self::Error> {
+    pub fn get_year(&mut self) -> u16 {
         let dr = self.regs.dr.read();
-        let year = bcd2_decode(dr.yt().bits(), dr.yu().bits());
-        Ok(year as u16)
+        bcd2_decode(dr.yt().bits(), dr.yu().bits()) as u16
     }
 
-    fn get_date(&mut self) -> Result<NaiveDate, Self::Error> {
-        let day = self.get_day()?;
-        let month = self.get_month()?;
-        let year = self.get_year()?;
-
-        Ok(NaiveDate::from_ymd(year.into(), month.into(), day.into()))
+    pub fn get_date(&mut self) -> NaiveDate {
+        NaiveDate::from_ymd(
+            self.get_year().into(),
+            self.get_month().into(),
+            self.get_day().into(),
+        )
     }
 
-    fn get_datetime(&mut self) -> Result<NaiveDateTime, Self::Error> {
-        self.set_24h_fmt();
-
-        let day = self.get_day()?;
-        let month = self.get_month()?;
-        let year = self.get_year()?;
-
-        let seconds = self.get_seconds()?;
-        let minutes = self.get_minutes()?;
-        let hours = hours_to_u8(self.get_hours()?)?;
-
-        Ok(
-            NaiveDate::from_ymd(year.into(), month.into(), day.into()).and_hms(
-                hours.into(),
-                minutes.into(),
-                seconds.into(),
-            ),
+    pub fn get_datetime(&mut self) -> NaiveDateTime {
+        NaiveDate::from_ymd(
+            self.get_year().into(),
+            self.get_month().into(),
+            self.get_day().into(),
+        )
+        .and_hms(
+            self.get_hours().into(),
+            self.get_minutes().into(),
+            self.get_seconds().into(),
         )
     }
 }
@@ -875,19 +851,3 @@ fn bcd2_encode(word: u32) -> Result<(u8, u8), Error> {
 fn bcd2_decode(fst: u8, snd: u8) -> u32 {
     (fst * 10 + snd).into()
 }
-
-fn hours_to_register(hours: Hours) -> Result<(u8, u8), Error> {
-    match hours {
-        Hours::H24(h) => Ok(bcd2_encode(h as u32))?,
-        Hours::AM(h) => Ok(bcd2_encode((h - 1) as u32))?,
-        Hours::PM(h) => Ok(bcd2_encode((h + 11) as u32))?,
-    }
-}
-
-fn hours_to_u8(hours: Hours) -> Result<u8, Error> {
-    if let Hours::H24(h) = hours {
-        Ok(h)
-    } else {
-        Err(Error::InvalidInputData)
-    }
-}
