@@ -1,4 +1,5 @@
-//! Serial Peripheral Interface (SPI) bus. Implements traits from `embedded-hal`.
+//! Serial Peripheral Interface (SPI) bus. Provides APIs to configure, read, and write from
+//! SPI, with blocking, nonblocking, and DMA functionality.
 
 use core::{
     ops::Deref,
@@ -6,17 +7,28 @@ use core::{
     sync::atomic::{self, Ordering},
 };
 
-use embedded_hal::spi::{FullDuplex, Mode, Phase, Polarity};
+use cortex_m::interrupt::free;
+
+#[cfg(feature = "embedded-hal")]
+use embedded_hal::spi::FullDuplex;
+
+use nb::block;
 
 use crate::{
+    clocks::Clocks,
     pac::{self, RCC},
     rcc_en_reset,
-    traits::ClockCfg,
 };
 
 #[cfg(feature = "g0")]
 use crate::pac::dma as dma_p;
-#[cfg(any(feature = "f3", feature = "l4", feature = "g4", feature = "wb"))]
+#[cfg(any(
+    feature = "f3",
+    feature = "l4",
+    feature = "g4",
+    feature = "wb",
+    feature = "wl"
+))]
 use crate::pac::dma1 as dma_p;
 
 #[cfg(not(any(feature = "h7", feature = "f4", feature = "l5")))]
@@ -28,7 +40,7 @@ use cfg_if::cfg_if;
 
 /// SPI error
 #[non_exhaustive]
-#[derive(Debug)]
+#[derive(Copy, Clone, Debug)]
 pub enum Error {
     /// Overrun occurred
     Overrun,
@@ -39,7 +51,7 @@ pub enum Error {
 }
 
 /// Possible interrupt types. Enable these in CR2. Check and clear with SR. Clear in ?
-#[derive(Debug)]
+#[derive(Copy, Clone)]
 pub enum SpiInterrupt {
     /// Tx buffer empty (TXEIE)
     TxBufEmpty,
@@ -52,12 +64,32 @@ pub enum SpiInterrupt {
 #[derive(Clone, Copy)]
 pub enum SpiDevice {
     One,
-    #[cfg(not(any(feature = "f3x4", feature = "wb")))]
+    #[cfg(not(any(feature = "f3x4", feature = "wb", feature = "wl")))]
     Two,
-    #[cfg(not(any(feature = "f3x4", feature = "f410", feature = "g0", feature = "wb")))]
+    #[cfg(not(any(
+        feature = "f3x4",
+        feature = "f410",
+        feature = "g0",
+        feature = "wb",
+        feature = "wl"
+    )))]
     Three,
 }
 
+/// Set the factor to divide the APB clock by to set baud rate. Sets `SPI_CR1` register, `BR` field.
+#[derive(Copy, Clone)]
+#[repr(u8)]
+pub enum BaudRate {
+    Div2 = 0b000,
+    Div4 = 0b001,
+    Div8 = 0b010,
+    Div16 = 0b011,
+    Div32 = 0b100,
+    Div64 = 0b101,
+    Div128 = 0b110,
+    Div256 = 0b111,
+}
+
 #[derive(Clone, Copy, PartialEq)]
 /// Select the communication mode between.
 pub enum SpiCommMode {
@@ -91,8 +123,71 @@ pub enum SlaveSelect {
     HardwareOutDisable,
 }
 
+#[derive(Clone, Copy)]
+#[repr(u8)]
+/// Clock polarity
+pub enum SpiPolarity {
+    /// Clock signal low when idle
+    IdleLow = 0,
+    /// Clock signal high when idle
+    IdleHigh = 1,
+}
+
+#[derive(Clone, Copy)]
+#[repr(u8)]
+/// Clock phase
+pub enum SpiPhase {
+    /// Data in "captured" on the first clock transition
+    CaptureOnFirstTransition = 0,
+    /// Data in "captured" on the second clock transition
+    CaptureOnSecondTransition = 1,
+}
+
+/// SPI mode
+#[derive(Clone, Copy)]
+pub struct SpiMode {
+    /// Clock polarity
+    pub polarity: SpiPolarity,
+    /// Clock phase
+    pub phase: SpiPhase,
+}
+
+impl SpiMode {
+    /// Set Spi Mode 0: Idle low, capture on first transition.
+    pub fn mode0() -> Self {
+        Self {
+            polarity: SpiPolarity::IdleLow,
+            phase: SpiPhase::CaptureOnFirstTransition,
+        }
+    }
+
+    /// Set Spi Mode 1: Idle low, capture on second transition.
+    pub fn mode1() -> Self {
+        Self {
+            polarity: SpiPolarity::IdleLow,
+            phase: SpiPhase::CaptureOnSecondTransition,
+        }
+    }
+
+    /// Set Spi Mode 2: Idle high, capture on first transition.
+    pub fn mode2() -> Self {
+        Self {
+            polarity: SpiPolarity::IdleHigh,
+            phase: SpiPhase::CaptureOnFirstTransition,
+        }
+    }
+
+    /// Set Spi Mode 3: Idle high, capture on second transition.
+    pub fn mode3() -> Self {
+        Self {
+            polarity: SpiPolarity::IdleHigh,
+            phase: SpiPhase::CaptureOnSecondTransition,
+        }
+    }
+}
+
 pub struct SpiConfig {
-    pub mode: Mode,
+    pub mode: SpiMode,
     pub comm_mode: SpiCommMode,
     pub slave_select: SlaveSelect,
     // pub cs_delay: f32,
@@ -103,157 +198,118 @@ pub struct SpiConfig {
 impl Default for SpiConfig {
     fn default() -> Self {
         Self {
-            mode: Mode {
-                // todo: What's a good default mode? Mode 1?
-                polarity: Polarity::IdleHigh,
-                phase: Phase::CaptureOnFirstTransition,
-            },
+            mode: SpiMode::mode0(),
             comm_mode: SpiCommMode::FullDuplex,
             slave_select: SlaveSelect::Software,
         }
     }
 }
 
-/// Represents an Serial Peripheral Interface (SPI) peripheral.
-pub struct Spi<S> {
-    regs: S,
+/// Represents a Serial Peripheral Interface (SPI) peripheral.
+pub struct Spi<R> {
+    regs: R,
     device: SpiDevice,
     cfg: SpiConfig,
 }
 
-impl<S> Spi<S>
+impl<R> Spi<R>
 where
-    S: Deref<Target = pac::spi1::RegisterBlock>,
+    R: Deref<Target = pac::spi1::RegisterBlock>,
 {
     /// Configures the SPI peripheral to operate in full duplex master mode
-    pub fn new<C: ClockCfg>(
-        regs: S,
-        device: SpiDevice,
-        cfg: SpiConfig,
-        freq: u32,
-        clocks: &C,
-        rcc: &mut RCC,
-    ) -> Self {
-        match device {
-            SpiDevice::One => {
-                #[cfg(not(feature = "f301"))] // todo: Not sure what's going on  here.
-                rcc_en_reset!(apb2, spi1, rcc);
-            }
-            #[cfg(not(any(feature = "f3x4", feature = "wb")))]
-            SpiDevice::Two => {
-                rcc_en_reset!(apb1, spi2, rcc);
-            }
-            #[cfg(not(any(feature = "f3x4", feature = "f410", feature = "g0", feature = "wb")))]
-            SpiDevice::Three => {
-                cfg_if! {
-                    // Note `sp3en` mixed with `spi3rst`; why we can't use the usual macro.
-                    if #[cfg(any(feature = "l4x3", feature = "l5"))] {
-                        rcc.apb1enr1.modify(|_, w| w.sp3en().set_bit());
-                        rcc.apb1rstr1.modify(|_, w| w.spi3rst().set_bit());
-                        rcc.apb1rstr1.modify(|_, w| w.spi3rst().clear_bit());
-                    } else {
-                        rcc_en_reset!(apb1, spi3, rcc);
+    pub fn new(regs: R, device: SpiDevice, cfg: SpiConfig, baud_rate: BaudRate) -> Self {
+        free(|cs| {
+            let mut rcc = unsafe { &(*RCC::ptr()) };
+
+            match device {
+                SpiDevice::One => {
+                    #[cfg(not(feature = "f301"))] // todo: Not sure what's going on  here.
+                    rcc_en_reset!(apb2, spi1, rcc);
+                }
+                #[cfg(not(any(feature = "f3x4", feature = "wb", feature = "wl")))]
+                SpiDevice::Two => {
+                    rcc_en_reset!(apb1, spi2, rcc);
+                }
+                #[cfg(not(any(
+                    feature = "f3x4",
+                    feature = "f410",
+                    feature = "g0",
+                    feature = "wb",
+                    feature = "wl"
+                )))]
+                SpiDevice::Three => {
+                    cfg_if! {
+                        // Note `sp3en` mixed with `spi3rst`; why we can't use the usual macro.
+                        if #[cfg(any(feature = "l4x3", feature = "l5"))] {
+                            rcc.apb1enr1.modify(|_, w| w.sp3en().set_bit());
+                            rcc.apb1rstr1.modify(|_, w| w.spi3rst().set_bit());
+                            rcc.apb1rstr1.modify(|_, w| w.spi3rst().clear_bit());
+                        } else {
+                            rcc_en_reset!(apb1, spi3, rcc);
+                        }
                     }
                 }
             }
-        }
+        });
 
         cfg_if! {
             if #[cfg(feature = "h7")] {
                   // Disable SS output
                 regs.cfg2.write(|w| w.ssoe().disabled());
 
-                let config: Config = config.into();
-
-                let spi_freq = freq;
-                let spi_ker_ck = match Self::kernel_clk(clocks) {
-                    Some(ker_hz) => ker_hz.0,
-                    _ => panic!("$SPX kernel clock not running!")
-                };
-                let mbr = match spi_ker_ck / spi_freq {
-                    0 => unreachable!(),
-                    1..=2 => MBR::DIV2,
-                    3..=5 => MBR::DIV4,
-                    6..=11 => MBR::DIV8,
-                    12..=23 => MBR::DIV16,
-                    24..=47 => MBR::DIV32,
-                    48..=95 => MBR::DIV64,
-                    96..=191 => MBR::DIV128,
-                    _ => MBR::DIV256,
-                };
-                regs.cfg1.modify(|_, w| {
-                    w.mbr()
-                        .variant(mbr) // master baud rate
-                });
-                spi!(DSIZE, spi, $TY); // modify CFG1 for DSIZE
+                regs.cfg1.modify(|_, w| w.mbr().bits(baud_rate as u8));
+                // spi!(DSIZE, spi, $TY); // modify CFG1 for DSIZE
 
                 // ssi: select slave = master mode
                 regs.cr1.write(|w| w.ssi().slave_not_selected());
 
+                // todo: Come back to this.
                 // Calculate the CS->transaction cycle delay bits.
-                let (start_cycle_delay, interdata_cycle_delay) = {
-                    let mut delay: u32 = (config.cs_delay * spi_freq as f32) as u32;
-
-                    // If the cs-delay is specified as non-zero, add 1 to the delay cycles
-                    // before truncation to an integer to ensure that we have at least as
-                    // many cycles as required.
-                    if config.cs_delay > 0.0_f32 {
-                        delay += 1;
-                    }
-
-                    if delay > 0xF {
-                        delay = 0xF;
-                    }
-
-                    // If CS suspends while data is inactive, we also require an
-                    // "inter-data" delay.
-                    if config.suspend_when_inactive {
-                        (delay as u8, delay as u8)
-                    } else {
-                        (delay as u8, 0_u8)
-                    }
-                };
+                // let (start_cycle_delay, interdata_cycle_delay) = {
+                //     let mut delay: u32 = (config.cs_delay * spi_freq as f32) as u32;
+                //
+                //     // If the cs-delay is specified as non-zero, add 1 to the delay cycles
+                //     // before truncation to an integer to ensure that we have at least as
+                //     // many cycles as required.
+                //     if config.cs_delay > 0.0_f32 {
+                //         delay += 1;
+                //     }
+                //
+                //     if delay > 0xF {
+                //         delay = 0xF;
+                //     }
+                //
+                //     // If CS suspends while data is inactive, we also require an
+                //     // "inter-data" delay.
+                //     if cfg.suspend_when_inactive {
+                //         (delay as u8, delay as u8)
+                //     } else {
+                //         (delay as u8, 0_u8)
+                //     }
+                // };
 
 
                 // mstr: master configuration
                 // lsbfrst: MSB first
                 // comm: full-duplex
+                // todo: Flesh this out.
                 regs.cfg2.write(|w| {
-                    w.cpha()
-                        .bit(config.mode.phase ==
-                             Phase::CaptureOnSecondTransition)
-                        .cpol()
-                        .bit(config.mode.polarity == Polarity::IdleHigh)
-                        .master()
-                        .master()
-                        .lsbfrst()
-                        .msbfirst()
-                        .ssom()
-                        .bit(config.suspend_when_inactive)
-                        .ssm()
-                        .bit(config.managed_cs == false)
-                        .ssoe()
-                        .bit(config.managed_cs == true)
-                        .mssi()
-                        .bits(start_cycle_delay)
-                        .midi()
-                        .bits(interdata_cycle_delay)
-                        .ioswp()
-                        .bit(config.swap_miso_mosi == true)
-                        .comm()
-                        .variant(communication_mode)
+                    w.cpha().bit(cfg.mode.phase as u8 != 0);
+                        w.cpol().bit(cfg.mode.polarity as u8 != 0);
+                        w.master().master();
+                        w.lsbfrst().msbfirst()
+                        // w.ssom().bit(config.suspend_when_inactive);
+                        // w.ssm().bit(config.managed_cs == false);
+                        // w.ssoe().bit(config.managed_cs == true);
+                        // w.mssi().bits(start_cycle_delay);
+                        // w.midi().bits(interdata_cycle_delay);
+                        // w.ioswp().bit(config.swap_miso_mosi == true)
+                        // w.comm().variant(communication_mode);
                 });
 
                 // spe: enable the SPI bus
                 regs.cr1.write(|w| w.ssi().slave_not_selected().spe().enabled());
             } else {
-                let fclk = match device {
-                    SpiDevice::One => clocks.apb2(),
-                    _ => clocks.apb1(),
-                };
-
-                let br = Self::compute_baud_rate(fclk, freq);
-
                 // L44 RM, section 40.4.7: Configuration of SPI
                 // The configuration procedure is almost the same for master and slave. For specific mode
                 // setups, follow the dedicated sections. When a standard communication is to be initialized,
@@ -265,13 +321,13 @@ where
                 // 2. Write to the SPI_CR1 register:
                 regs.cr1.modify(|_, w| unsafe {
                     // a) Configure the serial clock baud rate using the BR[2:0] bits (Note: 4)
-                    w.br().bits(br);
+                    w.br().bits(baud_rate as u8);
                     // b) Configure the CPOL and CPHA bits combination to define one of the four
                     // relationships between the data transfer and the serial clock (CPHA must be
                     // cleared in NSSP mode). (Note: 2 - except the case when CRC is enabled at TI
                     // mode).
-                    w.cpol().bit(cfg.mode.polarity == Polarity::IdleHigh);
-                    w.cpha().bit(cfg.mode.phase == Phase::CaptureOnSecondTransition);
+                    w.cpol().bit(cfg.mode.polarity as u8 != 0);
+                    w.cpha().bit(cfg.mode.phase as u8 != 0);
                     // c) Select simplex or half-duplex mode by configuring RXONLY or BIDIMODE and
                     // BIDIOE (RXONLY and BIDIMODE can't be set at the same time).
                     w.bidimode().bit(cfg.comm_mode == SpiCommMode::HalfDuplex);
@@ -324,39 +380,22 @@ where
     }
 
     /// Change the baud rate of the SPI
-    pub fn reclock<F, C: ClockCfg>(&mut self, freq: u32, clocks: C) {
+    pub fn reclock(&mut self, baud_rate: BaudRate) {
         self.regs.cr1.modify(|_, w| w.spe().clear_bit());
 
-        let fclk = match self.device {
-            SpiDevice::One => clocks.apb2(),
-            _ => clocks.apb1(),
-        };
-
-        let br = Self::compute_baud_rate(fclk, freq);
-
-        self.regs.cr1.modify(|_, w| {
-            unsafe {
-                w.br().bits(br);
-            }
+        #[cfg(not(feature = "h7"))]
+        self.regs.cr1.modify(|_, w| unsafe {
+            w.br().bits(baud_rate as u8);
             w.spe().set_bit()
         });
-    }
 
-    /// Compute value for the baud rate register (BR). BR is specified as
-    /// fraction offpclk (eg apb1 or apb2). Use the divider that will provide
-    /// a speed closest to the one requested.
-    fn compute_baud_rate(clocks: u32, freq: u32) -> u8 {
-        // todo: Check that this is the same across MCUs.
-        match clocks / freq {
-            0..=2 => 0b000,    // fpclk / 2
-            3..=6 => 0b001,    // /4
-            7..=11 => 0b010,   // /8
-            12..=23 => 0b011,  // /16
-            24..=39 => 0b100,  // /32
-            40..=95 => 0b101,  // /64
-            96..=191 => 0b110, // /128
-            _ => 0b111,        // /256
-        }
+        #[cfg(feature = "h7")]
+        self.regs
+            .cfg1
+            .modify(|_, w| unsafe { w.mbr().bits(baud_rate as u8) });
+
+        #[cfg(feature = "h7")]
+        self.regs.cr1.modify(|_, w| w.spe().set_bit());
     }
 
     /// L44 RM, section 40.4.9: "Procedure for disabling the SPI"
@@ -366,32 +405,145 @@ where
     /// modes the disable procedure is the only way to stop continuous communication running.
     pub fn disable(&mut self) {
         // The correct disable procedure is (except when receive only mode is used):
-        // 1. Wait until FTLVL[1:0] = 00 (no more data to transmit).
-        #[cfg(not(feature = "f4"))]
-        while self.regs.sr.read().ftlvl().bits() != 0 {}
-        // 2. Wait until BSY=0 (the last data frame is processed).
-        while self.regs.sr.read().bsy().bit_is_set() {}
-        // 3. Disable the SPI (SPE=0).
-        // todo: Instructions say to stop SPI (including to close DMA comms), but this breaks non-DMA writes, which assume
-        // todo SPI is enabled, the way we structure things.
-        self.regs.cr1.modify(|_, w| w.spe().clear_bit());
-        // 4. Read data until FRLVL[1:0] = 00 (read all the received data).
-        #[cfg(not(feature = "f4"))]
-        while self.regs.sr.read().frlvl().bits() != 0 {
-            unsafe { ptr::read_volatile(&self.regs.dr as *const _ as *const u8) };
+        cfg_if! {
+            if #[cfg(feature = "h7")] {
+                // 1. Wait until TXC=1 and/or EOT=1 (no more data to transmit and last data frame sent).
+                // When CRC is used, it is sent automatically after the last data in the block is processed.
+                // TXC/EOT is set when CRC frame is completed in this case. When a transmission is
+                // suspended the software has to wait till CSTART bit is cleared.
+                while self.regs.sr.read().txc().bit_is_clear() {}
+                while self.regs.sr.read().eot().bit_is_clear() {}
+                // 2. Read all RxFIFO data (until RXWNE=0 and RXPLVL=00)
+                while self.regs.sr.read().rxwne().bit_is_set() || self.regs.sr.read().rxplvl().bits() != 0  {
+                    unsafe { ptr::read_volatile(&self.regs.rxdr as *const _ as *const u8) };
+                }
+                // 3. Disable the SPI (SPE=0).
+                self.regs.cr1.modify(|_, w| w.spe().clear_bit());
+            } else {
+                 // 1. Wait until FTLVL[1:0] = 00 (no more data to transmit).
+                #[cfg(not(feature = "f4"))]
+                while self.regs.sr.read().ftlvl().bits() != 0 {}
+                // 2. Wait until BSY=0 (the last data frame is processed).
+                while self.regs.sr.read().bsy().bit_is_set() {}
+                // 3. Disable the SPI (SPE=0).
+                // todo: Instructions say to stop SPI (including to close DMA comms), but this breaks non-DMA writes, which assume
+                // todo SPI is enabled, the way we structure things.
+                self.regs.cr1.modify(|_, w| w.spe().clear_bit());
+                // 4. Read data until FRLVL[1:0] = 00 (read all the received data).
+                #[cfg(not(feature = "f4"))]
+                while self.regs.sr.read().frlvl().bits() != 0 {
+                    unsafe { ptr::read_volatile(&self.regs.dr as *const _ as *const u8) };
+                }
+            }
         }
     }
 
+    /// Read a single byte if available, or block until it's available.
+    /// See L44 RM, section 40.4.9: Data transmission and reception procedures.
+    pub fn read(&mut self) -> nb::Result<u8, Error> {
+        let sr = self.regs.sr.read();
+
+        cfg_if! {
+            if #[cfg(feature = "h7")] {
+                let crce = sr.crce().bit_is_set();
+                let not_empty = sr.rxp().bit_is_set();
+            } else {
+                let crce = sr.crcerr().bit_is_set();
+                let not_empty = sr.rxne().bit_is_set();
+            }
+        }
+
+        if sr.ovr().bit_is_set() {
+            Err(nb::Error::Other(Error::Overrun))
+        } else if sr.modf().bit_is_set() {
+            Err(nb::Error::Other(Error::ModeFault))
+        } else if crce {
+            Err(nb::Error::Other(Error::Crc))
+        } else if not_empty {
+            #[cfg(feature = "h7")]
+            // todo: note: H7 can support words beyond u8. (Can others too?)
+            let result = unsafe { ptr::read_volatile(&self.regs.rxdr as *const _ as *const u8) };
+            #[cfg(not(feature = "h7"))]
+            let result = unsafe { ptr::read_volatile(&self.regs.dr as *const _ as *const u8) };
+            Ok(result)
+        } else {
+            Err(nb::Error::WouldBlock)
+        }
+    }
+
+    /// Write a single byte if available, or block until it's available.
+    /// See L44 RM, section 40.4.9: Data transmission and reception procedures.
+    pub fn write_one(&mut self, byte: u8) -> nb::Result<(), Error> {
+        let sr = self.regs.sr.read();
+
+        cfg_if! {
+            if #[cfg(feature = "h7")] {
+                let crce = sr.crce().bit_is_set();
+                let rdy = sr.txp().bit_is_set();
+                let rdy = sr.txp().bit_is_set();
+            } else {
+                let crce = sr.crcerr().bit_is_set();
+                let rdy = sr.txe().bit_is_set();
+            }
+        }
+
+        if sr.ovr().bit_is_set() {
+            Err(nb::Error::Other(Error::Overrun))
+        } else if sr.modf().bit_is_set() {
+            Err(nb::Error::Other(Error::ModeFault))
+        } else if crce {
+            Err(nb::Error::Other(Error::Crc))
+        } else if rdy {
+            cfg_if! {
+                if #[cfg(feature = "h7")] {
+                    // todo: note: H7 can support words beyond u8. (Can others too?)
+                    unsafe { ptr::write_volatile(&self.regs.txdr as *const _ as *mut u8, byte) };
+                    // write CSTART to start a transaction in master mode
+                    self.regs.cr1.modify(|_, w| w.cstart().started());
+                }
+                 else {
+                    unsafe { ptr::write_volatile(&self.regs.dr as *const _ as *mut u8, byte) };
+                }
+            }
+            Ok(())
+        } else {
+            Err(nb::Error::WouldBlock)
+        }
+    }
+
+    /// Write multiple bytes on the SPI line, blocking until complete.
+    /// See L44 RM, section 40.4.9: Data transmission and reception procedures.
+    pub fn write(&mut self, words: &[u8]) -> Result<(), Error> {
+        for word in words {
+            nb::block!(self.write_one(word.clone()))?;
+            nb::block!(self.read())?;
+        }
+
+        Ok(())
+    }
+
+    /// Read multiple bytes to a buffer, blocking until complete.
+    /// See L44 RM, section 40.4.9: Data transmission and reception procedures.
+    pub fn transfer<'w>(&mut self, words: &'w mut [u8]) -> Result<(), Error> {
+        for word in words.iter_mut() {
+            nb::block!(self.write_one(word.clone()))?;
+            *word = nb::block!(self.read())?;
+        }
+
+        Ok(())
+    }
+
     #[cfg(not(any(feature = "g0", feature = "h7", feature = "f4", feature = "l5")))]
     /// Transmit data using DMA. See L44 RM, section 40.4.9: Communication using DMA.
     /// Note that the `channel` argument has no effect on F3 and L4.
-    pub fn write_dma<D>(&mut self, buf: &[u8], channel: DmaChannel, dma: &mut Dma<D>)
+    pub unsafe fn write_dma<D>(&mut self, buf: &[u8], channel: DmaChannel, dma: &mut Dma<D>)
     where
         D: Deref<Target = dma_p::RegisterBlock>,
     {
         // Static write and read buffers?
         let (ptr, len) = (buf.as_ptr(), buf.len());
 
+        // todo: Accept u16 words too.
         // todo: Pri and Circular as args?
 
         // A DMA access is requested when the TXE or RXNE enable bit in the SPIx_CR2 register is
@@ -410,9 +562,15 @@ where
         #[cfg(any(feature = "f3", feature = "l4"))]
         let tx_channel = match self.device {
             SpiDevice::One => DmaInput::Spi1Tx.dma1_channel(),
-            #[cfg(not(any(feature = "f3x4", feature = "wb")))]
+            #[cfg(not(any(feature = "f3x4", feature = "wb", feature = "wl")))]
             SpiDevice::Two => DmaInput::Spi2Tx.dma1_channel(),
-            #[cfg(not(any(feature = "f3x4", feature = "f410", feature = "g0", feature = "wb")))]
+            #[cfg(not(any(
+                feature = "f3x4",
+                feature = "f410",
+                feature = "g0",
+                feature = "wb",
+                feature = "wl"
+            )))]
             SpiDevice::Three => panic!(
                 "DMA on SPI3 is not supported. If it is for your MCU, please submit an issue \
                 or PR on Github."
@@ -474,10 +632,11 @@ where
     #[cfg(not(any(feature = "g0", feature = "h7", feature = "f4", feature = "l5")))]
     /// Receive data using DMA. See L44 RM, section 40.4.9: Communication using DMA.
     /// Note taht the `channel` argument has no effect on F3 and L4.
-    pub fn read_dma<D>(&mut self, buf: &mut [u8], channel: DmaChannel, dma: &mut Dma<D>)
+    pub unsafe fn read_dma<D>(&mut self, buf: &mut [u8], channel: DmaChannel, dma: &mut Dma<D>)
     where
         D: Deref<Target = dma_p::RegisterBlock>,
     {
+        // todo: Accept u16 words too.
         let (ptr, len) = (buf.as_mut_ptr(), buf.len());
 
         // atomic::compiler_fence(Ordering::Release);  // todo ?
@@ -487,9 +646,15 @@ where
         #[cfg(any(feature = "f3", feature = "l4"))]
         let channel = match self.device {
             SpiDevice::One => DmaInput::Spi1Rx.dma1_channel(),
-            #[cfg(not(any(feature = "f3x4", feature = "wb")))]
+            #[cfg(not(any(feature = "f3x4", feature = "wb", feature = "wl")))]
             SpiDevice::Two => DmaInput::Spi2Rx.dma1_channel(),
-            #[cfg(not(any(feature = "f3x4", feature = "f410", feature = "g0", feature = "wb")))]
+            #[cfg(not(any(
+                feature = "f3x4",
+                feature = "f410",
+                feature = "g0",
+                feature = "wb",
+                feature = "wl"
+            )))]
             _ => panic!(
                 "DMA on SPI3 is not supported. If it is for your MCU, please submit an issue \
                 or PR on Github."
@@ -499,9 +664,15 @@ where
         #[cfg(feature = "l4")]
         match self.device {
             SpiDevice::One => dma.channel_select(DmaInput::Spi1Rx),
-            #[cfg(not(any(feature = "f3x4", feature = "wb")))]
+            #[cfg(not(any(feature = "f3x4", feature = "wb", feature = "wl")))]
             SpiDevice::Two => dma.channel_select(DmaInput::Spi2Rx),
-            #[cfg(not(any(feature = "f3x4", feature = "f410", feature = "g0", feature = "wb")))]
+            #[cfg(not(any(
+                feature = "f3x4",
+                feature = "f410",
+                feature = "g0",
+                feature = "wb",
+                feature = "wl"
+            )))]
             _ => unimplemented!(),
         };
 
@@ -549,109 +720,46 @@ where
         })
     }
 
-    /// Enable an interrupt
+    #[cfg(not(feature = "h7"))]
+    /// Enable an interrupt. Note that unlike on other peripherals, there's no explicit way to
+    /// clear these. RM: "Writing to the transmit data register always clears the TXE bit.
+    /// The TXE flag is set by hardware."
     pub fn enable_interrupt(&mut self, interrupt_type: SpiInterrupt) {
-        match interrupt_type {
-            SpiInterrupt::TxBufEmpty => self.regs.cr2.modify(|_, w| w.txeie().set_bit()),
-            SpiInterrupt::RxBufNotEmpty => self.regs.cr2.modify(|_, w| w.rxneie().set_bit()),
-            SpiInterrupt::Error => self.regs.cr2.modify(|_, w| w.errie().set_bit()),
-        }
+        self.regs.cr2.modify(|_, w| match interrupt_type {
+            SpiInterrupt::TxBufEmpty => w.txeie().set_bit(),
+            SpiInterrupt::RxBufNotEmpty => w.rxneie().set_bit(),
+            SpiInterrupt::Error => w.errie().set_bit(),
+        });
     }
-
-    // // todo: Not sure how to clear SPI interrupts. No ICR, and SR is read only?
-    // /// Clear an interrupt flag.
-    // pub fn clear_interrupt(&mut self, interrupt_type: SpiInterrupt) {
-    //     match interrupt_type {
-    //         SpiInterrupt::TxBufEmpty => self.regs.cr2.modify(|_, w| w.txeie.set_bit()),
-    //         SpiInterrupt::RxBufNotEmpty => self.regs.cr2.modify(|_, w| w.rxneie.set_bit()),
-    //         SpiInterrupt::Error => self.regs.cr2.modify(|_, w| w.erreie.set_bit()),
-    //     }
-    // }
 }
 
-impl<S> FullDuplex<u8> for Spi<S>
+#[cfg(feature = "embedded-hal")]
+#[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
+impl<R> FullDuplex<u8> for Spi<R>
 where
-    S: Deref<Target = pac::spi1::RegisterBlock>,
+    R: Deref<Target = pac::spi1::RegisterBlock>,
 {
     type Error = Error;
 
-    /// See L44 RM, section 40.4.9: Data transmission and reception procedures.
     fn read(&mut self) -> nb::Result<u8, Error> {
-        let sr = self.regs.sr.read();
-
-        cfg_if! {
-            if #[cfg(feature = "h7")] {
-                let crce = sr.crce().bit_is_set();
-                let not_empty = sr.rxp().bit_is_set();
-            } else {
-                let crce = sr.crcerr().bit_is_set();
-                let not_empty = sr.rxne().bit_is_set();
-            }
-        }
-
-        if sr.ovr().bit_is_set() {
-            Err(nb::Error::Other(Error::Overrun))
-        } else if sr.modf().bit_is_set() {
-            Err(nb::Error::Other(Error::ModeFault))
-        } else if crce {
-            Err(nb::Error::Other(Error::Crc))
-        } else if not_empty {
-            #[cfg(feature = "h7")]
-            // todo: note: H7 can support words beyond u8. (Can others too?)
-            let result = unsafe { ptr::read_volatile(&self.regs.rxdr as *const _ as *const u8) };
-            #[cfg(not(feature = "h7"))]
-            let result = unsafe { ptr::read_volatile(&self.regs.dr as *const _ as *const u8) };
-            Ok(result)
-        } else {
-            Err(nb::Error::WouldBlock)
-        }
+        Spi::read(self)
     }
 
     fn send(&mut self, byte: u8) -> nb::Result<(), Error> {
-        let sr = self.regs.sr.read();
-
-        cfg_if! {
-            if #[cfg(feature = "h7")] {
-                let crce = sr.crce().bit_is_set();
-                let rdy = sr.txp().bit_is_set();
-                let rdy = sr.txp().bit_is_set();
-            } else {
-                let crce = sr.crcerr().bit_is_set();
-                let rdy = sr.txe().bit_is_set();
-            }
-        }
-
-        if sr.ovr().bit_is_set() {
-            Err(nb::Error::Other(Error::Overrun))
-        } else if sr.modf().bit_is_set() {
-            Err(nb::Error::Other(Error::ModeFault))
-        } else if crce {
-            Err(nb::Error::Other(Error::Crc))
-        } else if rdy {
-            cfg_if! {
-                if #[cfg(feature = "h7")] {
-                    // todo: note: H7 can support words beyond u8. (Can others too?)
-                    unsafe { ptr::write_volatile(&self.regs.txdr as *const _ as *mut u8, byte) };
-                    // write CSTART to start a transaction in master mode
-                    self.regs.cr1.modify(|_, w| w.cstart().started());
-                }
-                 else {
-                    unsafe { ptr::write_volatile(&self.regs.dr as *const _ as *mut u8, byte) };
-                }
-            }
-            Ok(())
-        } else {
-            Err(nb::Error::WouldBlock)
-        }
+        Spi::write_one(self, byte)
     }
 }
 
-impl<S> embedded_hal::blocking::spi::transfer::Default<u8> for Spi<S> where
-    S: Deref<Target = pac::spi1::RegisterBlock>
+#[cfg(feature = "embedded-hal")]
+#[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
+impl<R> embedded_hal::blocking::spi::transfer::Default<u8> for Spi<R> where
+    R: Deref<Target = pac::spi1::RegisterBlock>
 {
 }
 
-impl<S> embedded_hal::blocking::spi::write::Default<u8> for Spi<S> where
-    S: Deref<Target = pac::spi1::RegisterBlock>
+#[cfg(feature = "embedded-hal")]
+#[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
+impl<R> embedded_hal::blocking::spi::write::Default<u8> for Spi<R> where
+    R: Deref<Target = pac::spi1::RegisterBlock>
 {
 }
