@@ -1,7 +1,13 @@
-//! Timers. Includes initialization, countdown functionality, interrupts, and PWM features.
+//! Provides support for timers. Includes initialization, countdown functionality, interrupts,
+//! and PWM features.
+//!
+//! Low-power timers (LPTIM) are not yet supported.
 
 use num_traits::float::Float;
 
+use cortex_m::interrupt::free;
+
+#[cfg(feature = "embedded-hal")]
 use embedded_hal::{
     blocking::delay::{DelayMs, DelayUs},
     timer::{CountDown, Periodic},
@@ -12,9 +18,9 @@ use void::Void;
 // todo: LPTIM (low-power timers)
 
 use crate::{
+    clocks::Clocks,
     pac::{self, RCC},
     rcc_en_reset,
-    traits::ClockCfg,
 };
 
 use cfg_if::cfg_if;
@@ -69,14 +75,13 @@ pub enum Alignment {
     Center3,
 }
 
-// todo: TimerChannel, to avoid input conflicts?
 /// Timer channel
 #[derive(Clone, Copy)]
-pub enum Channel {
-    One,
-    Two,
-    Three,
-    Four,
+pub enum TimChannel {
+    C1,
+    C2,
+    C3,
+    C4,
 }
 
 /// Timer count direction
@@ -88,7 +93,8 @@ pub enum CountDir {
 }
 
 /// Capture/Compare selection.
-/// This bit-field defines the direction of the channel (input/output) as well as the used input.
+/// This field defines the direction of the channel (input/output) as well as the used input.
+/// It affects the TIMx_CCMR1 register, CCxS fields.
 #[repr(u8)]
 #[derive(Clone, Copy)]
 pub enum CaptureCompare {
@@ -203,9 +209,14 @@ macro_rules! hal {
         impl Timer<pac::$TIMX> {
             paste! {
                 /// Configures a TIM peripheral as a periodic count down timer
-                pub fn [<new_ $tim>]<C: ClockCfg>(tim: pac::$TIMX, freq: f32, clocks: &C, rcc: &mut RCC) -> Self {
-                    // `freq` is in Hz.
-                    rcc_en_reset!([<apb $apb>], $tim, rcc);
+                pub fn [<new_ $tim>](tim: pac::$TIMX, freq: f32, clocks: &Clocks) -> Self {
+
+                    free(|cs| {
+                        let mut rcc = unsafe { &(*RCC::ptr()) };
+
+                        // `freq` is in Hz.
+                        rcc_en_reset!([<apb $apb>], $tim, rcc);
+                    });
 
                     let clock_speed = match $apb {
                         1 => clocks.apb1_timer(),
@@ -229,20 +240,20 @@ macro_rules! hal {
             /// Starts listening for an `event`. Used to enable interrupts.
             pub fn enable_interrupt(&mut self, interrupt: TimerInterrupt) {
                 match interrupt {
-                        TimerInterrupt::Update => self.tim.dier.modify(|_, w| w.uie().set_bit()),
-                        // TimerInterrupt::Trigger => self.tim.dier.modify(|_, w| w.tie().set_bit()),
-                        // TimerInterrupt::CaptureCompare1 => self.tim.dier.modify(|_, w| w.cc1ie().set_bit()),
-                        // TimerInterrupt::CaptureCompare2 => self.tim.dier.modify(|_, w| w.cc2ie().set_bit()),
-                        // TimerInterrupt::CaptureCompare3 => self.tim.dier.modify(|_, w| w.cc3ie().set_bit()),
-                        // TimerInterrupt::CaptureCompare4 => self.tim.dier.modify(|_, w| w.cc4ie().set_bit()),
-                        // TimerInterrupt::UpdateDma => self.tim.dier.modify(|_, w| w.ude().set_bit()),
-                        // TimerInterrupt::TriggerDma => self.tim.dier.modify(|_, w| w.tde().set_bit()),
-                        // TimerInterrupt::CaptureCompare1Dma => self.tim.dier.modify(|_, w| w.cc1de().set_bit()),
-                        // TimerInterrupt::CaptureCompare2Dma => self.tim.dier.modify(|_, w| w.ccd2de().set_bit()),
-                        // TimerInterrupt::CaptureCompare3Dma => self.tim.dier.modify(|_, w| w.cc3de().set_bit()),
-                        // TimerInterrupt::CaptureCompare4Dma => self.tim.dier.modify(|_, w| w.cc4de().set_bit()),
-                        // todo: Only DIER is in PAC. PAC BUG? Only avail on some timers?
-                        _ => unimplemented!("TODO TEMP PROBLEMS"),
+                    TimerInterrupt::Update => self.tim.dier.modify(|_, w| w.uie().set_bit()),
+                    // TimerInterrupt::Trigger => self.tim.dier.modify(|_, w| w.tie().set_bit()),
+                    // TimerInterrupt::CaptureCompare1 => self.tim.dier.modify(|_, w| w.cc1ie().set_bit()),
+                    // TimerInterrupt::CaptureCompare2 => self.tim.dier.modify(|_, w| w.cc2ie().set_bit()),
+                    // TimerInterrupt::CaptureCompare3 => self.tim.dier.modify(|_, w| w.cc3ie().set_bit()),
+                    // TimerInterrupt::CaptureCompare4 => self.tim.dier.modify(|_, w| w.cc4ie().set_bit()),
+                    // TimerInterrupt::UpdateDma => self.tim.dier.modify(|_, w| w.ude().set_bit()),
+                    // TimerInterrupt::TriggerDma => self.tim.dier.modify(|_, w| w.tde().set_bit()),
+                    // TimerInterrupt::CaptureCompare1Dma => self.tim.dier.modify(|_, w| w.cc1de().set_bit()),
+                    // TimerInterrupt::CaptureCompare2Dma => self.tim.dier.modify(|_, w| w.ccd2de().set_bit()),
+                    // TimerInterrupt::CaptureCompare3Dma => self.tim.dier.modify(|_, w| w.cc3de().set_bit()),
+                    // TimerInterrupt::CaptureCompare4Dma => self.tim.dier.modify(|_, w| w.cc4de().set_bit()),
+                    // todo: Only DIER is in PAC. PAC BUG? Only avail on some timers?
+                    _ => unimplemented!("TODO TEMP PROBLEMS"),
                 }
             }
 
@@ -286,11 +297,11 @@ macro_rules! hal {
             /// in the constructor. If you use `center` aligned PWM, make sure to
             /// enter twice the freq you normally would.
             pub fn set_freq(&mut self, freq: f32) -> Result<(), ValueError> {
-               // todo: Take into account settings like Center alignment, and
-               // todo the `tim1sw` bit in RCC CFGR3, which change how the
-               // todo freq behaves. Center alignment halves the frequency;
-               // todo: Double `freq` here to compensate.
-               let (psc, arr) = calc_freq_vals(freq, self.clock_speed)?;
+                // todo: Take into account settings like Center alignment, and
+                // todo the `tim1sw` bit in RCC CFGR3, which change how the
+                // todo freq behaves. Center alignment halves the frequency;
+                // todo: Double `freq` here to compensate.
+                let (psc, arr) = calc_freq_vals(freq, self.clock_speed)?;
 
                 self.tim.arr.write(|w| unsafe { w.bits(arr.into()) });
                 self.tim.psc.write(|w| unsafe { w.bits(psc.into()) });
@@ -321,24 +332,32 @@ macro_rules! hal {
             }
         }
 
+        #[cfg(feature = "embedded-hal")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
         impl DelayMs<u32> for Timer<pac::$TIMX> {
             fn delay_ms(&mut self, ms: u32) {
                 self.delay_us(ms as u32 * 1_000);
             }
         }
 
+        #[cfg(feature = "embedded-hal")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
         impl DelayMs<u16> for Timer<pac::$TIMX> {
             fn delay_ms(&mut self, ms: u16) {
                 self.delay_us(ms as u32 * 1_000);
             }
         }
 
+        #[cfg(feature = "embedded-hal")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
         impl DelayMs<u8> for Timer<pac::$TIMX> {
             fn delay_ms(&mut self, ms: u8) {
                 self.delay_us(ms as u32 * 1_000);
             }
         }
 
+        #[cfg(feature = "embedded-hal")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
         impl DelayUs<u32> for Timer<pac::$TIMX> {
             fn delay_us(&mut self, us: u32) {
                 self.set_freq(1. / (us as f32 * 1_000.)).ok();
@@ -349,26 +368,34 @@ macro_rules! hal {
             }
         }
 
+        #[cfg(feature = "embedded-hal")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
         impl DelayUs<u16> for Timer<pac::$TIMX> {
             fn delay_us(&mut self, us: u16) {
                 self.delay_us(us as u32);
             }
         }
 
+        #[cfg(feature = "embedded-hal")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
         impl DelayUs<u8> for Timer<pac::$TIMX> {
             fn delay_us(&mut self, us: u8) {
                 self.delay_us(us as u32);
             }
         }
 
-         impl Periodic for Timer<pac::$TIMX> {}
+        #[cfg(feature = "embedded-hal")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
+        impl Periodic for Timer<pac::$TIMX> {}
 
+        #[cfg(feature = "embedded-hal")]
+        #[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
         impl CountDown for Timer<pac::$TIMX> {
             type Time = f32;
 
             fn start<T>(&mut self, timeout: T)
             where
-                T: Into<f32>,  // Hz.
+                T: Into<f32>, // Hz.
             {
                 self.disable();
 
@@ -395,7 +422,7 @@ macro_rules! hal {
                 }
             }
         }
-    }
+    };
 }
 
 /// Calculate values required to set the timer frequency: `PSC` and `ARR`. This can be
@@ -440,11 +467,13 @@ macro_rules! pwm_features {
             /// Enables basic PWM output
             pub fn enable_pwm_output(
                 &mut self,
-                channel: Channel,
+                channel: TimChannel,
                 compare: OutputCompare,
                 dir: CountDir,
                 duty: f32,
             ) {
+                // todo: duty as an f32 is good from an API perspective, but forces the
+                // todo use of software floats on non-FPU MCUs. How should we handle this?
                 self.set_preload(channel, true);
                 self.set_output_compare(channel, compare);
                 self.set_duty(channel, (self.get_max_duty() as f32 * duty) as $res);
@@ -456,7 +485,7 @@ macro_rules! pwm_features {
             /// L4 RM, section 26.3.8
             pub fn _enable_pwm_input(
                 &mut self,
-                channel: Channel,
+                channel: TimChannel,
                 compare: OutputCompare,
                 dir: CountDir,
                 duty: f32,
@@ -501,9 +530,9 @@ macro_rules! pwm_features {
             // todo: more advanced PWM modes. Asymmetric, combined, center-aligned etc.
 
             /// Set Output Compare Mode. See docs on the `OutputCompare` enum.
-            pub fn set_output_compare(&mut self, channel: Channel, mode: OutputCompare) {
+            pub fn set_output_compare(&mut self, channel: TimChannel, mode: OutputCompare) {
                 match channel {
-                    Channel::One => {
+                    TimChannel::C1 => {
                         self.tim
                             .ccmr1_output()
                             .modify(|_, w| unsafe { w.oc1m().bits(mode as u8) });
@@ -514,7 +543,7 @@ macro_rules! pwm_features {
                             .ccmr1_output()
                             .modify(|_, w| w.oc1m_3().bit(mode.left_bit()));
                     }
-                    Channel::Two => {
+                    TimChannel::C2 => {
                         self.tim
                             .ccmr1_output()
                             .modify(|_, w| unsafe { w.oc1m().bits(mode as u8) });
@@ -523,7 +552,7 @@ macro_rules! pwm_features {
                             .ccmr1_output()
                             .modify(|_, w| w.oc1m_3().bit(mode.left_bit()));
                     }
-                    Channel::Three => {
+                    TimChannel::C3 => {
                         self.tim
                             .ccmr1_output()
                             .modify(|_, w| unsafe { w.oc1m().bits(mode as u8) });
@@ -532,7 +561,7 @@ macro_rules! pwm_features {
                             .ccmr1_output()
                             .modify(|_, w| w.oc1m_3().bit(mode.left_bit()));
                     }
-                    Channel::Four => {
+                    TimChannel::C4 => {
                         self.tim
                             .ccmr2_output()
                             .modify(|_, w| unsafe { w.oc4m().bits(mode as u8) });
@@ -546,58 +575,61 @@ macro_rules! pwm_features {
 
             /// Return the set duty period for a given channel. Divide by `get_max_duty()`
             /// to find the portion of the duty cycle used.
-            pub fn get_duty(&self, channel: Channel) -> $res {
+            pub fn get_duty(&self, channel: TimChannel) -> $res {
                 cfg_if! {
                     if #[cfg(feature = "g0")] {
                         match channel {
-                            Channel::One => self.tim.ccr1.read().bits(),
-                            Channel::Two => self.tim.ccr2.read().bits(),
-                            Channel::Three => self.tim.ccr3.read().bits(),
-                            Channel::Four => self.tim.ccr4.read().bits(),
+                            // todo: This isn't right!!
+                            TimChannel::C1 => self.tim.ccr1.read().bits(),
+                            TimChannel::C2 => self.tim.ccr2.read().bits(),
+                            TimChannel::C3 => self.tim.ccr3.read().bits(),
+                            TimChannel::C4 => self.tim.ccr4.read().bits(),
                         }
-                    } else if #[cfg(feature = "g4")] {
+                    } else if #[cfg(any(feature = "g4", feature = "wb", feature = "wl"))] {
                         match channel {
-                            Channel::One => self.tim.ccr1.read().ccr1().bits(),
-                            Channel::Two => self.tim.ccr2.read().ccr2().bits(),
-                            Channel::Three => self.tim.ccr3.read().ccr3().bits(),
-                            Channel::Four => self.tim.ccr4.read().ccr4().bits(),
+                            TimChannel::C1 => self.tim.ccr1.read().ccr1().bits(),
+                            TimChannel::C2 => self.tim.ccr2.read().ccr2().bits(),
+                            TimChannel::C3 => self.tim.ccr3.read().ccr3().bits(),
+                            TimChannel::C4 => self.tim.ccr4.read().ccr4().bits(),
                         }
                     } else {
                         match channel {
-                            Channel::One => self.tim.ccr1.read().ccr().bits(),
-                            Channel::Two => self.tim.ccr2.read().ccr().bits(),
-                            Channel::Three => self.tim.ccr3.read().ccr().bits(),
-                            Channel::Four => self.tim.ccr4.read().ccr().bits(),
+                            TimChannel::C1 => self.tim.ccr1.read().ccr().bits(),
+                            TimChannel::C2 => self.tim.ccr2.read().ccr().bits(),
+                            TimChannel::C3 => self.tim.ccr3.read().ccr().bits(),
+                            TimChannel::C4 => self.tim.ccr4.read().ccr().bits(),
                         }
                     }
                 }
             }
 
-            /// Set the duty cycle, as a portion of `get_max_duty()`.
-            pub fn set_duty(&mut self, channel: Channel, duty: $res) {
+            /// Set the duty cycle, as a portion of ARR (`get_max_duty()`). Note that this
+            /// needs to be re-run if you change ARR at any point.
+            pub fn set_duty(&mut self, channel: TimChannel, duty: $res) {
                 cfg_if! {
                     if #[cfg(feature = "g0")] {
                         match channel {
-                            Channel::One => self.tim.ccr1.read().bits(),
-                            Channel::Two => self.tim.ccr2.read().bits(),
-                            Channel::Three => self.tim.ccr3.read().bits(),
-                            Channel::Four => self.tim.ccr4.read().bits(),
+                            // todo: This isn't right!!
+                            TimChannel::C1 => self.tim.ccr1.read().bits(),
+                            TimChannel::C2 => self.tim.ccr2.read().bits(),
+                            TimChannel::C3 => self.tim.ccr3.read().bits(),
+                            TimChannel::C4 => self.tim.ccr4.read().bits(),
                         };
-                    } else if #[cfg(feature = "g4")] {
+                    } else if #[cfg(any(feature = "g4", feature = "wb", feature = "wl"))] {
                         unsafe {
                             match channel {
-                                Channel::One => self.tim.ccr1.write(|w| w.ccr1().bits(duty)),
-                                Channel::Two => self.tim.ccr2.write(|w| w.ccr2().bits(duty)),
-                                Channel::Three => self.tim.ccr3.write(|w| w.ccr3().bits(duty)),
-                                Channel::Four => self.tim.ccr4.write(|w| w.ccr4().bits(duty)),
+                                TimChannel::C1 => self.tim.ccr1.write(|w| w.ccr1().bits(duty)),
+                                TimChannel::C2 => self.tim.ccr2.write(|w| w.ccr2().bits(duty)),
+                                TimChannel::C3 => self.tim.ccr3.write(|w| w.ccr3().bits(duty)),
+                                TimChannel::C4 => self.tim.ccr4.write(|w| w.ccr4().bits(duty)),
                             }
                         }
                     } else {
                         match channel {
-                            Channel::One => self.tim.ccr1.write(|w| w.ccr().bits(duty)),
-                            Channel::Two => self.tim.ccr2.write(|w| w.ccr().bits(duty)),
-                            Channel::Three => self.tim.ccr3.write(|w| w.ccr().bits(duty)),
-                            Channel::Four => self.tim.ccr4.write(|w| w.ccr().bits(duty)),
+                            TimChannel::C1 => self.tim.ccr1.write(|w| w.ccr().bits(duty)),
+                            TimChannel::C2 => self.tim.ccr2.write(|w| w.ccr().bits(duty)),
+                            TimChannel::C3 => self.tim.ccr3.write(|w| w.ccr().bits(duty)),
+                            TimChannel::C4 => self.tim.ccr4.write(|w| w.ccr().bits(duty)),
                         }
                     }
                 }
@@ -637,61 +669,61 @@ macro_rules! pwm_features {
             }
 
             /// Set output polarity. See docs on the `Polarity` enum.
-            pub fn set_polarity(&mut self, channel: Channel, polarity: Polarity) {
+            pub fn set_polarity(&mut self, channel: TimChannel, polarity: Polarity) {
                 match channel {
-                    Channel::One => self.tim.ccer.modify(|_, w| w.cc1p().bit(polarity.bit())),
-                    Channel::Two => self.tim.ccer.modify(|_, w| w.cc2p().bit(polarity.bit())),
-                    Channel::Three => self.tim.ccer.modify(|_, w| w.cc3p().bit(polarity.bit())),
-                    Channel::Four => self.tim.ccer.modify(|_, w| w.cc4p().bit(polarity.bit())),
+                    TimChannel::C1 => self.tim.ccer.modify(|_, w| w.cc1p().bit(polarity.bit())),
+                    TimChannel::C2 => self.tim.ccer.modify(|_, w| w.cc2p().bit(polarity.bit())),
+                    TimChannel::C3 => self.tim.ccer.modify(|_, w| w.cc3p().bit(polarity.bit())),
+                    TimChannel::C4 => self.tim.ccer.modify(|_, w| w.cc4p().bit(polarity.bit())),
                 }
             }
 
             /// Set complementary output polarity. See docs on the `Polarity` enum.
-            pub fn set_complementary_polarity(&mut self, channel: Channel, polarity: Polarity) {
+            pub fn set_complementary_polarity(&mut self, channel: TimChannel, polarity: Polarity) {
                 match channel {
-                    Channel::One => self.tim.ccer.modify(|_, w| w.cc1np().bit(polarity.bit())),
-                    Channel::Two => self.tim.ccer.modify(|_, w| w.cc2np().bit(polarity.bit())),
-                    Channel::Three => self.tim.ccer.modify(|_, w| w.cc3np().bit(polarity.bit())),
-                    Channel::Four => self.tim.ccer.modify(|_, w| w.cc4np().bit(polarity.bit())),
+                    TimChannel::C1 => self.tim.ccer.modify(|_, w| w.cc1np().bit(polarity.bit())),
+                    TimChannel::C2 => self.tim.ccer.modify(|_, w| w.cc2np().bit(polarity.bit())),
+                    TimChannel::C3 => self.tim.ccer.modify(|_, w| w.cc3np().bit(polarity.bit())),
+                    TimChannel::C4 => self.tim.ccer.modify(|_, w| w.cc4np().bit(polarity.bit())),
                 }
             }
             /// Disables capture compare on a specific channel.
-            pub fn disable_capture_compare(&mut self, channel: Channel) {
+            pub fn disable_capture_compare(&mut self, channel: TimChannel) {
                 match channel {
-                    Channel::One => self.tim.ccer.modify(|_, w| w.cc1e().clear_bit()),
-                    Channel::Two => self.tim.ccer.modify(|_, w| w.cc2e().clear_bit()),
-                    Channel::Three => self.tim.ccer.modify(|_, w| w.cc3e().clear_bit()),
-                    Channel::Four => self.tim.ccer.modify(|_, w| w.cc4e().clear_bit()),
+                    TimChannel::C1 => self.tim.ccer.modify(|_, w| w.cc1e().clear_bit()),
+                    TimChannel::C2 => self.tim.ccer.modify(|_, w| w.cc2e().clear_bit()),
+                    TimChannel::C3 => self.tim.ccer.modify(|_, w| w.cc3e().clear_bit()),
+                    TimChannel::C4 => self.tim.ccer.modify(|_, w| w.cc4e().clear_bit()),
                 }
             }
 
             /// Enables capture compare on a specific channel.
-            pub fn enable_capture_compare(&mut self, channel: Channel) {
+            pub fn enable_capture_compare(&mut self, channel: TimChannel) {
                 match channel {
-                    Channel::One => self.tim.ccer.modify(|_, w| w.cc1e().set_bit()),
-                    Channel::Two => self.tim.ccer.modify(|_, w| w.cc2e().set_bit()),
-                    Channel::Three => self.tim.ccer.modify(|_, w| w.cc3e().set_bit()),
-                    Channel::Four => self.tim.ccer.modify(|_, w| w.cc4e().set_bit()),
+                    TimChannel::C1 => self.tim.ccer.modify(|_, w| w.cc1e().set_bit()),
+                    TimChannel::C2 => self.tim.ccer.modify(|_, w| w.cc2e().set_bit()),
+                    TimChannel::C3 => self.tim.ccer.modify(|_, w| w.cc3e().set_bit()),
+                    TimChannel::C4 => self.tim.ccer.modify(|_, w| w.cc4e().set_bit()),
                 }
             }
 
             /// Set Capture Compare Mode. See docs on the `CaptureCompare` enum.
-            pub fn set_capture_compare(&mut self, channel: Channel, mode: CaptureCompare) {
+            pub fn set_capture_compare(&mut self, channel: TimChannel, mode: CaptureCompare) {
                 match channel {
                     // Note: CC1S bits are writable only when the channel is OFF (CC1E = 0 in TIMx_CCER)
-                    Channel::One => self
+                    TimChannel::C1 => self
                         .tim
                         .ccmr1_output()
                         .modify(unsafe { |_, w| w.cc1s().bits(mode as u8) }),
-                    Channel::Two => self
+                    TimChannel::C2 => self
                         .tim
                         .ccmr1_output()
                         .modify(unsafe { |_, w| w.cc2s().bits(mode as u8) }),
-                    Channel::Three => self
+                    TimChannel::C3 => self
                         .tim
                         .ccmr2_output()
                         .modify(unsafe { |_, w| w.cc3s().bits(mode as u8) }),
-                    Channel::Four => self
+                    TimChannel::C4 => self
                         .tim
                         .ccmr2_output()
                         .modify(unsafe { |_, w| w.cc4s().bits(mode as u8) }),
@@ -716,12 +748,12 @@ macro_rules! pwm_features {
             /// pulse mode (OPM bit set in TIMx_CR1 register). Else the behavior is not guaranteed.
             ///
             /// Setting preload is required to enable PWM.
-            pub fn set_preload(&mut self, channel: Channel, value: bool) {
+            pub fn set_preload(&mut self, channel: TimChannel, value: bool) {
                 match channel {
-                    Channel::One => self.tim.ccmr1_output().modify(|_, w| w.oc1pe().bit(value)),
-                    Channel::Two => self.tim.ccmr1_output().modify(|_, w| w.oc2pe().bit(value)),
-                    Channel::Three => self.tim.ccmr2_output().modify(|_, w| w.oc3pe().bit(value)),
-                    Channel::Four => self.tim.ccmr2_output().modify(|_, w| w.oc4pe().bit(value)),
+                    TimChannel::C1 => self.tim.ccmr1_output().modify(|_, w| w.oc1pe().bit(value)),
+                    TimChannel::C2 => self.tim.ccmr1_output().modify(|_, w| w.oc2pe().bit(value)),
+                    TimChannel::C3 => self.tim.ccmr2_output().modify(|_, w| w.oc3pe().bit(value)),
+                    TimChannel::C4 => self.tim.ccmr2_output().modify(|_, w| w.oc4pe().bit(value)),
                 }
 
                 // "As the preload registers are transferred to the shadow registers only when an update event
@@ -743,7 +775,8 @@ hal!(TIM1, tim1, 2);
     feature = "f4",
     feature = "l4",
     feature = "l5",
-    feature = "g0"
+    feature = "g0",
+    feature = "wl",  // todo temp!
 )))]
 pwm_features!(TIM1, u16);
 
@@ -763,15 +796,24 @@ cfg_if! {
 #[cfg(feature = "g4")]
 pwm_features!(TIM2, u16);
 
-#[cfg(not(any(feature = "l5", feature = "g070", feature = "g4", feature = "f410")))]
+#[cfg(not(any(
+    feature = "l5",
+    feature = "g070",
+    feature = "g4",
+    feature = "f410",
+    feature = "wb",
+    feature = "wl"
+)))]
 pwm_features!(TIM2, u32);
 
 #[cfg(not(any(
     feature = "f301",
     feature = "l4x1",
+    feature = "l412",
     feature = "l4x3",
     feature = "f410",
-    feature = "wb"
+    feature = "wb",
+    feature = "wl"
 )))]
 hal!(TIM3, tim3, 1);
 
@@ -782,6 +824,7 @@ hal!(TIM3, tim3, 1);
     feature = "f410",
     feature = "g0",
     feature = "wb",
+    feature = "wl"
 )))]
 pwm_features!(TIM3, u16);
 
@@ -795,11 +838,12 @@ cfg_if! {
         feature = "f410",
         feature = "l4x1",
         feature = "l4x2",
-        feature = "l412", // todo: 412 workaround
+        feature = "l412",
         feature = "l4x3",
         feature = "l552",
         feature = "g0",
         feature = "wb",
+        feature = "wl"
     )))] {
         hal!(TIM4, tim4, 1);
     }
@@ -812,11 +856,12 @@ cfg_if! {
         feature = "f410",
         feature = "l4x1",
         feature = "l4x2",
-        feature = "l412", // todo: 412 workaround
+        feature = "l412",
         feature = "l4x3",
         feature = "l5",
         feature = "g0",
         feature = "wb",
+        feature = "wl"
     )))] {
         pwm_features!(TIM4, u16);
     }
@@ -857,6 +902,7 @@ cfg_if! {
         feature = "g070",
         feature = "g030",
         feature = "wb",
+        feature = "wl"
     )))] {
         hal!(TIM6, tim6, 1);
     }
@@ -872,6 +918,7 @@ cfg_if! {
     feature = "g041",
     feature = "g030",
     feature = "wb",
+    feature = "wl"
 )))]
 hal!(TIM7, tim7, 1);
 
@@ -892,6 +939,8 @@ hal!(TIM8, tim8, 2);
     feature = "g031",
     feature = "g041",
     feature = "g030",
+    feature = "wb",
+    feature = "wl"
 )))]
 hal!(TIM15, tim15, 2);
 
@@ -902,7 +951,7 @@ cfg_if! {
     if #[cfg(not(any(
         feature = "l4x1",
         feature = "l4x2",
-        feature = "l412", // todo: 412 workaround
+        feature = "l412",
         feature = "l4x3",
         feature = "f4",
     )))] {
