@@ -1,5 +1,6 @@
 //! This module allows for serial communication using the STM32 USART module.
-//! Supports Embedded HAL `Read` and `Write` traits.
+//! Provides APIs to configure, read, and write from
+//! USART, with blocking, nonblocking, and DMA functionality.
 
 // todo: Synchronous mode.
 // todo: Auto baud
@@ -7,12 +8,14 @@
 // todo: Missing some features (like additional interrupts) on the USARTv3 peripheral . (L5, G etc)
 
 use crate::{
+    clocks::Clocks,
     pac::{self, RCC},
     rcc_en_reset,
-    traits::ClockCfg,
 };
 use core::ops::Deref;
 
+use cortex_m::interrupt::free;
+
 #[cfg(feature = "g0")]
 use crate::pac::dma as dma_p;
 #[cfg(any(feature = "f3", feature = "l4", feature = "g4", feature = "wb"))]
@@ -21,6 +24,7 @@ use crate::pac::dma1 as dma_p;
 #[cfg(not(any(feature = "h7", feature = "f4", feature = "l5")))]
 use crate::dma::{self, Dma, DmaChannel, DmaInput};
 
+#[cfg(feature = "embedded-hal")]
 use embedded_hal::{
     blocking,
     serial::{Read, Write},
@@ -49,18 +53,16 @@ pub enum Parity {
 }
 
 #[derive(Clone, Copy)]
-#[repr(u8)]
 /// The length of word to transmit and receive. (USART_CR1, M)
 pub enum WordLen {
-    W8 = 0b00,
-    W9 = 0b01,
-    W7 = 0b10,
+    W8,
+    W9,
+    W7,
 }
 
 impl WordLen {
     /// We use this function due to the M field being split into 2 separate bits.
     /// Returns M1 val, M0 val
-    /// todo: Make sure this isn't backwards.
     fn bits(&self) -> (u8, u8) {
         match self {
             Self::W8 => (0, 0),
@@ -138,69 +140,71 @@ impl Default for UsartConfig {
 }
 
 /// Represents the USART peripheral, for serial communications.
-pub struct Usart<U> {
-    regs: U,
+pub struct Usart<R> {
+    regs: R,
     device: UsartDevice,
     pub baud: u32,
     pub config: UsartConfig,
 }
 
-impl<U> Usart<U>
+impl<R> Usart<R>
 where
-    U: Deref<Target = pac::usart1::RegisterBlock>,
+    R: Deref<Target = pac::usart1::RegisterBlock>,
 {
-    pub fn new<C: ClockCfg>(
-        regs: U,
+    pub fn new(
+        regs: R,
         device: UsartDevice,
         baud: u32,
         config: UsartConfig,
-        clock_cfg: &C,
-        rcc: &mut RCC,
+        clock_cfg: &Clocks,
     ) -> Self {
-        match device {
-            UsartDevice::One => {
-                rcc_en_reset!(apb2, usart1, rcc);
-            }
-            #[cfg(not(feature = "wb"))]
-            UsartDevice::Two => {
-                cfg_if! {
-                    if #[cfg(not(feature = "f4"))] {
-                        rcc_en_reset!(apb1, usart2, rcc);
-                    } else {
-                        // Notice the `usart` vs `uart` asymmetry.
-                        rcc.apb1enr.modify(|_, w| w.usart2en().set_bit());
-                        rcc.apb1rstr.modify(|_, w| w.uart2rst().set_bit());
-                        rcc.apb1rstr.modify(|_, w| w.uart2rst().clear_bit());
+        free(|cs| {
+            let mut rcc = unsafe { &(*RCC::ptr()) };
+            match device {
+                UsartDevice::One => {
+                    rcc_en_reset!(apb2, usart1, rcc);
+                }
+                #[cfg(not(feature = "wb"))]
+                UsartDevice::Two => {
+                    cfg_if! {
+                        if #[cfg(not(feature = "f4"))] {
+                            rcc_en_reset!(apb1, usart2, rcc);
+                        } else {
+                            // Notice the `usart` vs `uart` asymmetry.
+                            rcc.apb1enr.modify(|_, w| w.usart2en().set_bit());
+                            rcc.apb1rstr.modify(|_, w| w.uart2rst().set_bit());
+                            rcc.apb1rstr.modify(|_, w| w.uart2rst().clear_bit());
+                        }
                     }
                 }
-            }
-            #[cfg(not(any(
-                feature = "f401",
-                feature = "f410",
-                feature = "f411",
-                feature = "f412",
-                feature = "f413",
-                feature = "l4x1",
-                feature = "g0",
-                feature = "wb",
-            )))]
-            UsartDevice::Three => {
-                cfg_if! {
-                    if #[cfg(not(feature = "f4"))] {
-                        rcc_en_reset!(apb1, usart3, rcc);
-                    } else {
-                        rcc.apb1enr.modify(|_, w| w.usart3en().set_bit());
-                        rcc.apb1rstr.modify(|_, w| w.uart3rst().set_bit());
-                        rcc.apb1rstr.modify(|_, w| w.uart3rst().clear_bit());
+                #[cfg(not(any(
+                    feature = "f401",
+                    feature = "f410",
+                    feature = "f411",
+                    feature = "f412",
+                    feature = "f413",
+                    feature = "l4x1",
+                    feature = "g0",
+                    feature = "wb",
+                )))]
+                UsartDevice::Three => {
+                    cfg_if! {
+                        if #[cfg(not(feature = "f4"))] {
+                            rcc_en_reset!(apb1, usart3, rcc);
+                        } else {
+                            rcc.apb1enr.modify(|_, w| w.usart3en().set_bit());
+                            rcc.apb1rstr.modify(|_, w| w.uart3rst().set_bit());
+                            rcc.apb1rstr.modify(|_, w| w.uart3rst().clear_bit());
+                        }
                     }
-                }
-            } // UsartDevice::Four => {
-              //     rcc_en_reset!(apb1, uart4, rcc);
-              // }
-              // UsartDevice::Five => {
-              //     rcc_en_reset!(apb1, usart5, rcc);
-              // }
-        }
+                } // UsartDevice::Four => {
+                  //     rcc_en_reset!(apb1, uart4, rcc);
+                  // }
+                  // UsartDevice::Five => {
+                  //     rcc_en_reset!(apb1, usart5, rcc);
+                  // }
+            }
+        });
 
         let mut result = Self {
             regs,
@@ -217,33 +221,33 @@ where
         #[cfg(not(any(feature = "f3", feature = "f4")))]
         let word_len_bits = result.config.word_len.bits();
 
+        // Set up transmission. See L44 RM, section 38.5.2: "Character Transmission Procedures".
+        // 1. Program the M bits in USART_CR1 to define the word length.
+
+        let word_len_bits = result.config.word_len.bits();
         result.regs.cr1.modify(|_, w| {
             w.over8().bit(result.config.oversampling as u8 != 0);
             w.pce().bit(result.config.parity != Parity::Disabled);
-            w.ps().bit(result.config.parity == Parity::EnabledOdd);
             cfg_if! {
-                if #[cfg(any(feature = "f3", feature = "f4"))] {
-                    return w.m().bit(result.config.word_len as u8 != 0);
-                } else {
+                if #[cfg(not(any(feature = "f3", feature = "f4")))] {
                     w.m1().bit(word_len_bits.0 != 0);
-                    return w.m0().bit(word_len_bits.1 != 0);
+                    w.m0().bit(word_len_bits.1 != 0);
+                    return w.ps().bit(result.config.parity == Parity::EnabledOdd);
+                } else {
+                    return w.ps().bit(result.config.parity == Parity::EnabledOdd);
                 }
             }
         });
 
-        // Set up transmission. See L44 RM, section 38.5.2: "Character Transmission Procedures".
-        // 1. Program the M bits in USART_CR1 to define the word length.
-        // todo: Make sure you aren't doing this backwards.
-
-        cfg_if! {
-            if #[cfg(any(feature = "f3", feature = "f4"))] {
-                result.regs.cr1.modify(|_, w| w.m().bit(result.config.word_len as u8 != 0));
-            } else {
-                let word_len_bits = result.config.word_len.bits();
-                result.regs.cr1.modify(|_, w| w.m1().bit(word_len_bits.0 != 0));
-                result.regs.cr1.modify(|_, w| w.m0().bit(word_len_bits.1 != 0));
-            }
-        }
+        // todo: Workaround due to a PAC bug, where M0 is missing.
+        #[cfg(any(feature = "f3", feature = "f4"))]
+        result.regs.cr1.write(|w| unsafe {
+            w.bits(
+                result.regs.cr1.read().bits()
+                    | ((word_len_bits.0 as u32) << 28)
+                    | ((word_len_bits.1 as u32) << 12),
+            )
+        });
 
         // 2. Select the desired baud rate using the USART_BRR register.
         result.set_baud(baud, clock_cfg);
@@ -256,7 +260,7 @@ where
         result.regs.cr1.modify(|_, w| w.ue().set_bit());
         // 5. Select DMA enable (DMAT[R]] in USART_CR3 if multibuffer communication is to take
         // place. Configure the DMA register as explained in multibuffer communication.
-        // (Handled in `enable_dma()`)
+        // (Handled in `read_dma()` and `write_dma()`)
         // 6. Set the TE bit in USART_CR1 to send an idle frame as first transmission.
         // 6. Set the RE bit USART_CR1. This enables the receiver which begins searching for a
         // start bit.
@@ -270,7 +274,7 @@ where
 
     /// Set the BAUD rate. Called during init, and can be called later to change BAUD
     /// during program execution.
-    pub fn set_baud<C: ClockCfg>(&mut self, baud: u32, clock_cfg: &C) {
+    pub fn set_baud(&mut self, baud: u32, clock_cfg: &Clocks) {
         let originally_enabled = self.regs.cr1.read().ue().bit_is_set();
 
         if originally_enabled {
@@ -385,26 +389,10 @@ where
         }
     }
 
-    #[cfg(any(feature = "f3", feature = "l4"))]
-    /// Enable use of DMA transmission for U[s]ART: (L44 RM, section 38.5.15)
-    pub fn enable_dma<D>(&mut self, dma: &mut Dma<D>)
-    where
-        D: Deref<Target = dma_p::RegisterBlock>,
-    {
-        // "DMA mode can be enabled for transmission by setting DMAT bit in the USART_CR3
-        // register. Data is loaded from a SRAM area configured using the DMA peripheral (refer to
-        // Section 11: Direct memory access controller (DMA) on page 295) to the USART_TDR
-        // register whenever the TXE bit is set."
-        self.regs.cr3.modify(|_, w| w.dmat().set_bit());
-        self.regs.cr3.modify(|_, w| w.dmar().set_bit());
-
-        // Note that we need neither channel select, nor multiplex for F3.
-    }
-
     #[cfg(not(any(feature = "g0", feature = "h7", feature = "f4", feature = "l5")))]
     /// Transmit data using DMA. (L44 RM, section 38.5.15)
     /// Note that the `channel` argument is only used on F3 and L4.
-    pub fn write_dma<D>(&mut self, buf: &[u8], channel: DmaChannel, dma: &mut Dma<D>)
+    pub unsafe fn write_dma<D>(&mut self, buf: &[u8], channel: DmaChannel, dma: &mut Dma<D>)
     where
         D: Deref<Target = dma_p::RegisterBlock>,
     {
@@ -443,6 +431,8 @@ where
             UsartDevice::Three => dma.channel_select(DmaInput::Usart3Tx),
         }
 
+        // Note that we need neither channel select, nor multiplex for F3.
+
         // todo: Pri and Circular as args?
 
         dma.cfg_channel(
@@ -484,12 +474,19 @@ where
         // disabling the USART or entering Stop mode. Software must wait until TC=1. The TC flag
         // remains cleared during all data transfers and it is set by hardware at the end of transmission
         // of the last frame.
+
+        // (this RM quote isn't part of the steps and section the others above are)
+        // "DMA mode can be enabled for transmission by setting DMAT bit in the USART_CR3
+        // register. Data is loaded from a SRAM area configured using the DMA peripheral (refer to
+        // Section 11: Direct memory access controller (DMA) on page 295) to the USART_TDR
+        // register whenever the TXE bit is set."
+        self.regs.cr3.modify(|_, w| w.dmat().set_bit());
     }
 
     #[cfg(not(any(feature = "g0", feature = "h7", feature = "f4", feature = "l5")))]
     /// Receive data using DMA. (L44 RM, section 38.5.15)
     /// Note that the `channel` argument is only used on F3 and L4.
-    pub fn read_dma<D>(&mut self, buf: &mut [u8], channel: DmaChannel, dma: &mut Dma<D>)
+    pub unsafe fn read_dma<D>(&mut self, buf: &mut [u8], channel: DmaChannel, dma: &mut Dma<D>)
     where
         D: Deref<Target = dma_p::RegisterBlock>,
     {
@@ -541,6 +538,8 @@ where
             Default::default(),
         );
 
+        self.regs.cr3.modify(|_, w| w.dmar().set_bit());
+
         // 5. Configure interrupt generation after half/ full transfer as required by the application.
         // todo (Let the user call `dma.setup_interrupt()`? But that requires they know
         // todo which channel of DMA usart tx is on.
@@ -668,6 +667,8 @@ pub enum Error {
     Parity,
 }
 
+#[cfg(feature = "embedded-hal")]
+#[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
 impl<U> Read<u8> for Usart<U>
 where
     U: Deref<Target = pac::usart1::RegisterBlock>,
@@ -697,9 +698,11 @@ where
     }
 }
 
-impl<U> Write<u8> for Usart<U>
+#[cfg(feature = "embedded-hal")]
+#[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
+impl<R> Write<u8> for Usart<R>
 where
-    U: Deref<Target = pac::usart1::RegisterBlock>,
+    R: Deref<Target = pac::usart1::RegisterBlock>,
 {
     type Error = Error;
 
@@ -745,9 +748,11 @@ where
     }
 }
 
-impl<U> blocking::serial::Write<u8> for Usart<U>
+#[cfg(feature = "embedded-hal")]
+#[cfg_attr(docsrs, doc(cfg(feature = "embedded-hal")))]
+impl<R> blocking::serial::Write<u8> for Usart<R>
 where
-    U: Deref<Target = pac::usart1::RegisterBlock>,
+    R: Deref<Target = pac::usart1::RegisterBlock>,
 {
     type Error = Error;
 
